@@ -267,42 +267,38 @@ class MetaModelUnStructuredComp(ExplicitComponent):
                                    wrt=tuple([name[0] for name in self._surrogate_input_names]),
                                    dct=dct)
 
-            # warn the user that if they don't explicitly set options for fd,
-            #   the defaults will be used
-            # get a list of approximated partials
-            declared_partials = set([
-                key for key, dct in iteritems(self._subjacs_info) if 'method' in dct
-                and dct['method']])
+        # Support for user declaring fd partials in a child class and assigning new defaults.
+        # We want a warning for all partials that were not explicitly declared.
+        declared_partials = set([
+            key for key, dct in iteritems(self._subjacs_info) if 'method' in dct
+            and dct['method']])
 
-            non_declared_partials = []
-            for of, n_of in self._surrogate_output_names:
-                has_derivs = False
-                surrogate = self._metadata(of).get('surrogate')
-                if surrogate:
-                    has_derivs = overrides_method('linearize', surrogate, SurrogateModel)
-                if not has_derivs:
-                    for wrt, n_wrt in self._surrogate_input_names:
-                        abs_key = rel_key2abs_key(self, (of, wrt))
-                        if abs_key not in declared_partials:
-                            non_declared_partials.append(abs_key)
-            if non_declared_partials:
-                msg = "Because the MetaModelUnStructuredComp '{}' uses a surrogate " \
-                      "which does not define a linearize method,\nOpenMDAO will use " \
-                      "finite differences to compute derivatives. Some of the derivatives " \
-                      "will be computed\nusing default finite difference " \
-                      "options because they were not explicitly declared.\n".format(self.name)
-                msg += "The derivatives computed using the defaults are:\n"
-                for abs_key in non_declared_partials:
-                    msg += "    {}, {}\n".format(*abs_key)
-                simple_warning(msg, RuntimeWarning)
+        # Gather undeclared fd partials on surrogates that don't support analytic derivatives.
+        # While we do this, declare the missing ones.
+        non_declared_partials = []
+        for of, _ in self._surrogate_output_names:
+            surrogate = self._metadata(of).get('surrogate')
+            if surrogate and not overrides_method('linearize', surrogate, SurrogateModel):
+                wrt_list = [name[0] for name in self._surrogate_input_names]
+                self._approx_partials(of=of, wrt=wrt_list, method='fd')
 
-            for out_name, out_shape in self._surrogate_output_names:
-                surrogate = self._metadata(out_name).get('surrogate')
-                if surrogate and not overrides_method('linearize', surrogate, SurrogateModel):
-                    self._approx_partials(of=out_name,
-                                          wrt=[name[0] for name in self._surrogate_input_names],
-                                          method='fd')
-                    self._get_approx_scheme('fd')
+                for wrt in wrt_list:
+                    abs_key = rel_key2abs_key(self, (of, wrt))
+                    if abs_key not in declared_partials:
+                        non_declared_partials.append(abs_key)
+
+        if non_declared_partials:
+            self._get_approx_scheme('fd')
+
+            msg = "Because the MetaModelUnStructuredComp '{}' uses a surrogate " \
+                  "which does not define a linearize method,\nOpenMDAO will use " \
+                  "finite differences to compute derivatives. Some of the derivatives " \
+                  "will be computed\nusing default finite difference " \
+                  "options because they were not explicitly declared.\n".format(self.name)
+            msg += "The derivatives computed using the defaults are:\n"
+            for abs_key in non_declared_partials:
+                msg += "    {}, {}\n".format(*abs_key)
+            simple_warning(msg, RuntimeWarning)
 
     def check_config(self, logger):
         """
