@@ -249,10 +249,10 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
                 if num_sample[fi] is None:
                     num_sample[fi] = len(val)
                 elif len(val) != num_sample[fi]:
-                    msg = "MultiFiMetaModelUnStructured: Each variable must have the same number"\
-                          " of training points. Expected {0} but found {1} "\
-                          "points for '{2}'."\
-                          .format(num_sample[fi], len(val), name)
+                    msg = "{}: Each variable must have the same number"\
+                          " of training points. Expected {} but found {} "\
+                          "points for '{}'."\
+                          .format(self.msginfo, num_sample[fi], len(val), name)
                     raise RuntimeError(msg)
 
         inputs = [np.zeros((num_sample[fi], self._input_sizes[fi]))
@@ -294,8 +294,8 @@ class MultiFiMetaModelUnStructuredComp(MetaModelUnStructuredComp):
 
             surrogate = self._metadata(name_root).get('surrogate')
             if surrogate is None:
-                msg = "MultiFiMetaModelUnStructured '{}': No surrogate specified for output '{}'"
-                raise RuntimeError(msg.format(self.pathname, name_root))
+                msg = "{}: No surrogate specified for output '{}'"
+                raise RuntimeError(msg.format(self.msginfo, name_root))
             else:
                 surrogate.train_multifi(inputs, self._training_output[name])
 
