@@ -1,3 +1,6 @@
+"""
+Unit tests for the BalanceComp.
+"""
 from __future__ import print_function, division, absolute_import
 
 import os
@@ -7,29 +10,24 @@ import warnings
 import numpy as np
 from numpy.testing import assert_almost_equal
 
-from openmdao.api import Problem, Group, IndepVarComp, ExecComp, \
-    NewtonSolver, DirectSolver
-from openmdao.api import BalanceComp
+import openmdao.api as om
 
 
 class TestBalanceComp(unittest.TestCase):
 
     def test_scalar_example(self):
 
-        prob = Problem(model=Group())
-
-        bal = BalanceComp()
+        prob = om.Problem()
 
+        bal = om.BalanceComp()
         bal.add_balance('x', val=1.0)
 
-        tgt = IndepVarComp(name='y_tgt', val=2)
+        tgt = om.IndepVarComp(name='y_tgt', val=2)
 
-        exec_comp = ExecComp('y=x**2')
+        exec_comp = om.ExecComp('y=x**2')
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
-
         prob.model.add_subsystem(name='exec', subsys=exec_comp)
-
         prob.model.add_subsystem(name='balance', subsys=bal)
 
         prob.model.connect('y_tgt', 'balance.rhs:x')
@@ -50,8 +48,8 @@ class TestBalanceComp(unittest.TestCase):
             assert_almost_equal(cpd['balance'][of, wrt]['abs error'], 0.0, decimal=5)
 
         # set an actual solver, and re-setup. Then check derivatives at a converged point
-        prob.model.linear_solver = DirectSolver()
-        prob.model.nonlinear_solver = NewtonSolver()
+        prob.model.linear_solver = om.DirectSolver()
+        prob.model.nonlinear_solver = om.NewtonSolver()
 
         prob.setup()
 
@@ -66,26 +64,24 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_create_on_init(self):
 
-        prob = Problem(model=Group())
+        prob = om.Problem()
 
-        bal = BalanceComp('x', val=1.0)
+        bal = om.BalanceComp('x', val=1.0)
 
-        tgt = IndepVarComp(name='y_tgt', val=2)
+        tgt = om.IndepVarComp(name='y_tgt', val=2)
 
-        exec_comp = ExecComp('y=x**2')
+        exec_comp = om.ExecComp('y=x**2')
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
-
         prob.model.add_subsystem(name='exec', subsys=exec_comp)
-
         prob.model.add_subsystem(name='balance', subsys=bal)
 
         prob.model.connect('y_tgt', 'balance.rhs:x')
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver()
-        prob.model.nonlinear_solver = NewtonSolver()
+        prob.model.linear_solver = om.DirectSolver()
+        prob.model.nonlinear_solver = om.NewtonSolver()
 
         prob.setup()
 
@@ -103,13 +99,13 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_create_on_init_no_normalization(self):
 
-        prob = Problem(model=Group())
+        prob = om.Problem()
 
-        bal = BalanceComp('x', val=1.0, normalize=False)
+        bal = om.BalanceComp('x', val=1.0, normalize=False)
 
-        tgt = IndepVarComp(name='y_tgt', val=1.5)
+        tgt = om.IndepVarComp(name='y_tgt', val=1.5)
 
-        exec_comp = ExecComp('y=x**2')
+        exec_comp = om.ExecComp('y=x**2')
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -121,8 +117,8 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver()
-        prob.model.nonlinear_solver = NewtonSolver(iprint=0)
+        prob.model.linear_solver = om.DirectSolver()
+        prob.model.nonlinear_solver = om.NewtonSolver(iprint=0)
 
         prob.setup()
 
@@ -141,15 +137,15 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 100
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x', val=np.ones(n))
 
-        tgt = IndepVarComp(name='y_tgt', val=4*np.ones(n))
+        tgt = om.IndepVarComp(name='y_tgt', val=4*np.ones(n))
 
-        exec_comp = ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
+        exec_comp = om.ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -161,9 +157,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -182,15 +178,15 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 100
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x', val=np.ones(n), normalize=False)
 
-        tgt = IndepVarComp(name='y_tgt', val=1.7*np.ones(n))
+        tgt = om.IndepVarComp(name='y_tgt', val=1.7*np.ones(n))
 
-        exec_comp = ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
+        exec_comp = om.ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -202,9 +198,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -223,17 +219,17 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 100
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x', val=np.ones(n), use_mult=True)
 
-        tgt = IndepVarComp(name='y_tgt', val=4*np.ones(n))
+        tgt = om.IndepVarComp(name='y_tgt', val=4*np.ones(n))
 
-        mult_ivc = IndepVarComp(name='mult', val=2.0*np.ones(n))
+        mult_ivc = om.IndepVarComp(name='mult', val=2.0*np.ones(n))
 
-        exec_comp = ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
+        exec_comp = om.ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         prob.model.add_subsystem(name='mult_comp', subsys=mult_ivc, promotes_outputs=['mult'])
@@ -247,9 +243,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -272,13 +268,13 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 100
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp('x', val=np.ones(n), use_mult=True, mult_val=2.0)
+        bal = om.BalanceComp('x', val=np.ones(n), use_mult=True, mult_val=2.0)
 
-        tgt = IndepVarComp(name='y_tgt', val=4 * np.ones(n))
+        tgt = om.IndepVarComp(name='y_tgt', val=4 * np.ones(n))
 
-        exec_comp = ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
+        exec_comp = om.ExecComp('y=x**2', x={'value': np.ones(n)}, y={'value': np.ones(n)})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -290,9 +286,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -310,14 +306,14 @@ class TestBalanceComp(unittest.TestCase):
     def test_shape(self):
         n = 100
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
         bal.add_balance('x', shape=(n,))
 
-        tgt = IndepVarComp(name='y_tgt', val=4*np.ones(n))
+        tgt = om.IndepVarComp(name='y_tgt', val=4*np.ones(n))
 
-        exe = ExecComp('y=x**2', x=np.zeros(n), y=np.zeros(n))
+        exe = om.ExecComp('y=x**2', x=np.zeros(n), y=np.zeros(n))
 
-        model = Group()
+        model = om.Group()
 
         model.add_subsystem('tgt', tgt, promotes_outputs=['y_tgt'])
         model.add_subsystem('exe', exe)
@@ -327,10 +323,10 @@ class TestBalanceComp(unittest.TestCase):
         model.connect('bal.x', 'exe.x')
         model.connect('exe.y', 'bal.lhs:x')
 
-        model.linear_solver = DirectSolver(assemble_jac=True)
-        model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        model.linear_solver = om.DirectSolver(assemble_jac=True)
+        model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob = Problem(model)
+        prob = om.Problem(model)
         prob.setup()
 
         prob['bal.x'] = np.random.rand(n)
@@ -343,15 +339,15 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x')
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -363,9 +359,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup(force_alloc_complex=True)
 
@@ -384,15 +380,15 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x')
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
 
@@ -404,9 +400,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -425,16 +421,16 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        model=Group(assembled_jac_type='dense')
+        model=om.Group(assembled_jac_type='dense')
 
         def guess_function(inputs, outputs, residuals):
             outputs['x'] = np.sqrt(inputs['rhs:x'])
 
-        bal = BalanceComp('x', guess_func=guess_function)  # test guess_func as kwarg
+        bal = om.BalanceComp('x', guess_func=guess_function)  # test guess_func as kwarg
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         model.add_subsystem(name='exec', subsys=exec_comp)
@@ -444,10 +440,10 @@ class TestBalanceComp(unittest.TestCase):
         model.connect('balance.x', 'exec.x')
         model.connect('exec.y', 'balance.lhs:x')
 
-        model.linear_solver = DirectSolver(assemble_jac=True)
-        model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        model.linear_solver = om.DirectSolver(assemble_jac=True)
+        model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob = Problem(model)
+        prob = om.Problem(model)
         prob.setup()
 
         prob['balance.x'] = np.random.rand(n)
@@ -464,17 +460,17 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_scalar_with_guess_func_additional_input(self):
 
-        model = Group(assembled_jac_type='dense')
+        model = om.Group(assembled_jac_type='dense')
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
         bal.add_balance('x')
         bal.add_input('guess_x', val=0.0)
 
-        ivc = IndepVarComp()
+        ivc = om.IndepVarComp()
         ivc.add_output(name='y_tgt', val=4)
         ivc.add_output(name='guess_x', val=2.5)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         model.add_subsystem(name='ivc', subsys=ivc, promotes_outputs=['y_tgt', 'guess_x'])
         model.add_subsystem(name='exec', subsys=exec_comp)
@@ -485,10 +481,10 @@ class TestBalanceComp(unittest.TestCase):
         model.connect('balance.x', 'exec.x')
         model.connect('exec.y', 'balance.lhs:x')
 
-        model.linear_solver = DirectSolver(assemble_jac=True)
-        model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        model.linear_solver = om.DirectSolver(assemble_jac=True)
+        model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob = Problem(model)
+        prob = om.Problem(model)
         prob.setup()
 
         # run problem without a guess function
@@ -517,15 +513,15 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_scalar_guess_func_using_outputs(self):
 
-        model = Group()
+        model = om.Group()
 
-        ind = IndepVarComp()
+        ind = om.IndepVarComp()
         ind.add_output('a', 1)
         ind.add_output('b', -4)
         ind.add_output('c', 3)
 
-        lhs = ExecComp('lhs=-(a*x**2+b*x)')
-        bal = BalanceComp(name='x', rhs_name='c')
+        lhs = om.ExecComp('lhs=-(a*x**2+b*x)')
+        bal = om.BalanceComp(name='x', rhs_name='c')
 
         model.add_subsystem('ind_comp', ind, promotes_outputs=['a', 'b', 'c'])
         model.add_subsystem('lhs_comp', lhs, promotes_inputs=['a', 'b', 'x'])
@@ -533,12 +529,12 @@ class TestBalanceComp(unittest.TestCase):
 
         model.connect('lhs_comp.lhs', 'bal_comp.lhs:x')
 
-        model.linear_solver = DirectSolver()
-        model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        model.linear_solver = om.DirectSolver()
+        model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         # first verify behavior of the balance comp without the guess function
         # at initial conditions x=5, x=0 and x=-1
-        prob = Problem(model)
+        prob = om.Problem(model)
         prob.setup()
 
         # default solution with initial value of 5 is x=3.
@@ -584,11 +580,11 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp('x', rhs_val=4.0)
+        bal = om.BalanceComp('x', rhs_val=4.0)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='exec', subsys=exec_comp)
 
@@ -597,9 +593,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -618,17 +614,17 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x', use_mult=True)
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        mult_ivc = IndepVarComp(name='mult', val=2.0)
+        mult_ivc = om.IndepVarComp(name='mult', val=2.0)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         prob.model.add_subsystem(name='mult_comp', subsys=mult_ivc, promotes_outputs=['mult'])
@@ -642,9 +638,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -663,17 +659,17 @@ class TestBalanceComp(unittest.TestCase):
 
         n = 1
 
-        prob = Problem(model=Group(assembled_jac_type='dense'))
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
 
         bal.add_balance('x', use_mult=True, mult_name='MUL', lhs_name='XSQ', rhs_name='TARGETXSQ')
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        mult_ivc = IndepVarComp(name='mult', val=2.0)
+        mult_ivc = om.IndepVarComp(name='mult', val=2.0)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         prob.model.add_subsystem(name='mult_comp', subsys=mult_ivc, promotes_outputs=['mult'])
@@ -687,9 +683,9 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.XSQ')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
 
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
         prob.setup()
 
@@ -706,19 +702,18 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_feature_scalar(self):
         from numpy.testing import assert_almost_equal
-        from openmdao.api import Problem, Group, IndepVarComp, ExecComp, NewtonSolver, \
-            DirectSolver, BalanceComp
+        import openmdao.api as om
 
-        prob = Problem()
+        prob = om.Problem()
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
         bal.add_balance('x', use_mult=True)
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        mult_ivc = IndepVarComp(name='mult', val=2.0)
+        mult_ivc = om.IndepVarComp(name='mult', val=2.0)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         prob.model.add_subsystem(name='mult_comp', subsys=mult_ivc, promotes_outputs=['mult'])
@@ -730,10 +725,10 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob.setup(check=False)
+        prob.setup()
 
         # A reasonable initial guess to find the positive root.
         prob['balance.x'] = 1.0
@@ -744,17 +739,16 @@ class TestBalanceComp(unittest.TestCase):
 
     def test_feature_scalar_with_default_mult(self):
         from numpy.testing import assert_almost_equal
-        from openmdao.api import Problem, Group, IndepVarComp, ExecComp, NewtonSolver, \
-            DirectSolver, BalanceComp
+        import openmdao.api as om
 
-        prob = Problem()
+        prob = om.Problem()
 
-        bal = BalanceComp()
+        bal = om.BalanceComp()
         bal.add_balance('x', use_mult=True, mult_val=2.0)
 
-        tgt = IndepVarComp(name='y_tgt', val=4)
+        tgt = om.IndepVarComp(name='y_tgt', val=4)
 
-        exec_comp = ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
+        exec_comp = om.ExecComp('y=x**2', x={'value': 1}, y={'value': 1})
 
         prob.model.add_subsystem(name='target', subsys=tgt, promotes_outputs=['y_tgt'])
         prob.model.add_subsystem(name='exec', subsys=exec_comp)
@@ -764,10 +758,10 @@ class TestBalanceComp(unittest.TestCase):
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob.setup(check=False)
+        prob.setup()
 
         # A reasonable initial guess to find the positive root.
         prob['balance.x'] = 1.0
@@ -780,28 +774,28 @@ class TestBalanceComp(unittest.TestCase):
         import numpy as np
         from numpy.testing import assert_almost_equal
 
-        from openmdao.api import Problem, Group, ExecComp, NewtonSolver, DirectSolver, BalanceComp
+        import openmdao.api as om
 
         n = 100
 
-        prob = Problem()
+        prob = om.Problem()
 
-        exec_comp = ExecComp('y=b*x+c',
-                             b={'value': np.random.uniform(0.01,100, size=n)},
-                             c={'value': np.random.rand(n)},
-                             x={'value': np.zeros(n)},
-                             y={'value': np.ones(n)})
+        exec_comp = om.ExecComp('y=b*x+c',
+                                b={'value': np.random.uniform(0.01, 100, size=n)},
+                                c={'value': np.random.rand(n)},
+                                x={'value': np.zeros(n)},
+                                y={'value': np.ones(n)})
 
         prob.model.add_subsystem(name='exec', subsys=exec_comp)
-        prob.model.add_subsystem(name='balance', subsys=BalanceComp('x', val=np.ones(n)))
+        prob.model.add_subsystem(name='balance', subsys=om.BalanceComp('x', val=np.ones(n)))
 
         prob.model.connect('balance.x', 'exec.x')
         prob.model.connect('exec.y', 'balance.lhs:x')
 
-        prob.model.linear_solver = DirectSolver(assemble_jac=True)
-        prob.model.nonlinear_solver = NewtonSolver(maxiter=100, iprint=0)
+        prob.model.linear_solver = om.DirectSolver(assemble_jac=True)
+        prob.model.nonlinear_solver = om.NewtonSolver(maxiter=100, iprint=0)
 
-        prob.setup(check=False)
+        prob.setup()
 
         prob['balance.x'] = np.random.rand(n)
 
