@@ -265,7 +265,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: No valid expressions provided to ExecComp(): [].")
+                         "ExecComp (C1): No valid expressions provided to ExecComp(): [].")
 
     def test_colon_vars(self):
         prob = om.Problem()
@@ -273,7 +273,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: failed to compile expression 'y=foo:bar+1.'.")
+                         "ExecComp (C1): failed to compile expression 'y=foo:bar+1.'.")
 
     def test_bad_kwargs(self):
         prob = om.Problem()
@@ -281,7 +281,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: arg 'xx' in call to ExecComp() does not refer to any variable "
+                         "ExecComp (C1): arg 'xx' in call to ExecComp() does not refer to any variable "
                          "in the expressions ['y=x+1.']")
 
     def test_bad_kwargs_meta(self):
@@ -291,7 +291,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: the following metadata names were not recognized for "
+                         "ExecComp (C1): the following metadata names were not recognized for "
                          "variable 'x': ['high', 'low', 'val']")
 
     def test_name_collision_const(self):
@@ -300,7 +300,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: cannot assign to variable 'e' because it's already defined "
+                         "ExecComp (C1): cannot assign to variable 'e' because it's already defined "
                          "as an internal function or constant.")
 
     def test_name_collision_func(self):
@@ -309,7 +309,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: cannot assign to variable 'sin' because it's already defined "
+                         "ExecComp (C1): cannot assign to variable 'sin' because it's already defined "
                          "as an internal function or constant.")
 
     def test_func_as_rhs_var(self):
@@ -318,7 +318,7 @@ class TestExecComp(unittest.TestCase):
         with self.assertRaises(Exception) as context:
             prob.setup()
         self.assertEqual(str(context.exception),
-                         "C1: cannot use 'sin' as a variable because it's already defined "
+                         "ExecComp (C1): cannot use 'sin' as a variable because it's already defined "
                          "as an internal function.")
 
     def test_mixed_type(self):
@@ -399,7 +399,7 @@ class TestExecComp(unittest.TestCase):
                                                        units=2.0))
 
         self.assertEqual(str(cm.exception),
-                         "Value (2.0) of option 'units' has type 'float', "
+                         "ExecComp: Value (2.0) of option 'units' has type 'float', "
                          "but type 'str' was expected.")
 
     def test_units_varname_str(self):
@@ -425,7 +425,7 @@ class TestExecComp(unittest.TestCase):
             prob.setup()
 
         self.assertEqual(str(cm.exception),
-                         "C1: cannot use variable name 'units' because it's a reserved keyword.")
+                         "ExecComp (C1): cannot use variable name 'units' because it's a reserved keyword.")
 
     def test_common_units(self):
         # all variables in the ExecComp have the same units
@@ -468,7 +468,7 @@ class TestExecComp(unittest.TestCase):
             prob.setup()
 
         self.assertEqual(str(cm.exception),
-                         "C1: units of 'km' have been specified for variable 'x', but "
+                         "ExecComp (C1): units of 'km' have been specified for variable 'x', but "
                          "units of 'm' have been specified for the entire component.")
 
     def test_shape_and_value(self):
@@ -504,7 +504,7 @@ class TestExecComp(unittest.TestCase):
             p.setup()
 
         self.assertEqual(str(context.exception).replace('L,', ','),  # L on Windows
-                         "comp: shape of (5,) has been specified for variable 'x', "
+                         "ExecComp (comp): shape of (5,) has been specified for variable 'x', "
                          "but a value of shape (1,) has been provided.")
 
     def test_common_shape(self):
@@ -555,7 +555,7 @@ class TestExecComp(unittest.TestCase):
             p.setup()
 
         self.assertEqual(str(context.exception).replace('L,', ','),  # L on Windows
-                         "comp: shape of (10,) has been specified for variable 'y', "
+                         "ExecComp (comp): shape of (10,) has been specified for variable 'y', "
                          "but shape of (5,) has been specified for the entire component.")
 
     def test_common_shape_conflicting_value(self):
@@ -572,7 +572,7 @@ class TestExecComp(unittest.TestCase):
             p.setup()
 
         self.assertEqual(str(context.exception).replace('1L,', '1,'),  # 1L on Windows
-                         "comp: value of shape (1,) has been specified for variable 'x', "
+                         "ExecComp (comp): value of shape (1,) has been specified for variable 'x', "
                          "but shape of (5,) has been specified for the entire component.")
 
     def test_math(self):
@@ -592,6 +592,45 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, C1._outputs['y'], math.sin(2.0), 0.00001)
 
+    def test_np(self):
+        prob = om.Problem()
+        prob.model.add_subsystem('C1', om.ExecComp(['y1=sin(x)', 'y2=np.cos(x)'], x=2.0))
+        prob.setup()
+
+        with self.assertRaises(Exception) as context:
+          prob.run_model()
+
+        self.assertEqual(str(context.exception),
+            "ExecComp (C1): Error occurred evaluating 'y2=np.cos(x)'\n"
+            "    ExecComp supports a subset of numpy functions directly, without the 'np' prefix.\n"
+            "    'cos' is supported (See the documentation).")
+
+    def test_numpy(self):
+        prob = om.Problem()
+        prob.model.add_subsystem('C1', om.ExecComp('y=numpy.sin(x)', x=2.0))
+        prob.setup()
+
+        with self.assertRaises(Exception) as context:
+          prob.run_model()
+
+        self.assertEqual(str(context.exception),
+            "ExecComp (C1): Error occurred evaluating 'y=numpy.sin(x)'\n"
+            "    ExecComp supports a subset of numpy functions directly, without the 'numpy' prefix.\n"
+            "    'sin' is supported (See the documentation).")
+
+    def test_numpy_fft(self):
+        prob = om.Problem()
+        prob.model.add_subsystem('C1', om.ExecComp('y=numpy.fft(x)', x=2.0))
+        prob.setup()
+
+        with self.assertRaises(Exception) as context:
+          prob.run_model()
+
+        self.assertEqual(str(context.exception),
+            "ExecComp (C1): Error occurred evaluating 'y=numpy.fft(x)'\n"
+            "    ExecComp supports a subset of numpy functions directly, without the 'numpy' prefix.\n"
+            "    'fft' is not supported (See the documentation).")
+
     def test_array(self):
         prob = om.Problem()
         C1 = prob.model.add_subsystem('C1', om.ExecComp('y=x[1]',
@@ -770,27 +809,27 @@ class TestExecComp(unittest.TestCase):
 
         assert_rel_error(self, C1._jacobian['y', 'x'], expect, 0.00001)
 
-    def test_vectorize_error(self):
+    def test_has_diag_partials_error(self):
         p = om.Problem()
         model = p.model
         model.add_subsystem('indep', om.IndepVarComp('x', val=np.ones(3)))
         model.add_design_var('indep.x')
 
         mat = np.arange(15).reshape((3,5))
-        model.add_subsystem('comp', om.ExecComp('y=A.dot(x)', vectorize=True, A=mat, x=np.ones(5), y=np.ones(3)))
+        model.add_subsystem('comp', om.ExecComp('y=A.dot(x)', has_diag_partials=True, A=mat, x=np.ones(5), y=np.ones(3)))
         model.connect('indep.x', 'comp.x')
 
         with self.assertRaises(Exception) as context:
             p.setup()
         self.assertEqual(str(context.exception),
-                         "comp: vectorize is True but partial(y, A) is not square (shape=(3, 15)).")
+                         "ExecComp (comp): has_diag_partials is True but partial(y, A) is not square (shape=(3, 15)).")
 
-    def test_vectorize_shape_only(self):
+    def test_has_diag_partials_shape_only(self):
         p = om.Problem()
         model = p.model
         model.add_subsystem('indep', om.IndepVarComp('x', val=np.ones(5)))
 
-        model.add_subsystem('comp', om.ExecComp('y=3.0*x + 2.5', vectorize=True,
+        model.add_subsystem('comp', om.ExecComp('y=3.0*x + 2.5', has_diag_partials=True,
                                                 x={'shape': (5,)}, y={'shape': (5,)}))
         model.connect('indep.x', 'comp.x')
 
@@ -801,7 +840,69 @@ class TestExecComp(unittest.TestCase):
 
         assert_almost_equal(J, np.eye(5)*3., decimal=6)
 
-    def test_feature_vectorize(self):
+    def test_tags(self):
+        prob = om.Problem(model=om.Group())
+        prob.model.add_subsystem('indep', om.IndepVarComp('x', 100.0, units='cm'))
+        C1 = prob.model.add_subsystem('C1', om.ExecComp('y=x+z+1.',
+                                                     x={'value': 1.0, 'units': 'm', 'tags': 'tagx'},
+                                                     y={'units': 'm', 'tags': ['tagy','tagq']},
+                                                     z={'value': 2.0, 'tags': 'tagz'},
+                                                     ))
+        prob.model.connect('indep.x', 'C1.x')
+
+        prob.setup(check=False)
+
+        prob.set_solver_print(level=0)
+        prob.run_model()
+
+        # Inputs no tags
+        inputs = prob.model.list_inputs(values=False, out_stream=None)
+        self.assertEqual(sorted(inputs), [
+            ('C1.x', {}),
+            ('C1.z', {}),
+        ])
+
+        # Inputs with tags
+        inputs = prob.model.list_inputs(values=False, out_stream=None, tags="tagx")
+        self.assertEqual(sorted(inputs), [
+            ('C1.x', {}),
+        ])
+
+        # Inputs with multiple tags
+        inputs = prob.model.list_inputs(values=False, out_stream=None, tags=["tagx", "tagz"])
+        self.assertEqual(sorted(inputs), [
+            ('C1.x', {}),
+            ('C1.z', {}),
+        ])
+
+        # Inputs with tag that does not match
+        inputs = prob.model.list_inputs(values=False, out_stream=None, tags="tag_wrong")
+        self.assertEqual(sorted(inputs), [])
+
+        # Outputs no tags
+        outputs = prob.model.list_outputs(values=False, out_stream=None)
+        self.assertEqual(sorted(outputs), [
+            ('C1.y', {}),
+            ('indep.x', {}),
+        ])
+
+        # Outputs with tags
+        outputs = prob.model.list_outputs(values=False, out_stream=None, tags="tagy")
+        self.assertEqual(sorted(outputs), [
+            ('C1.y', {}),
+        ])
+
+        # Outputs with multiple tags
+        outputs = prob.model.list_outputs(values=False, out_stream=None, tags=["tagy", "tagx"])
+        self.assertEqual(sorted(outputs), [
+            ('C1.y', {}),
+        ])
+
+        # Outputs with tag that does not match
+        outputs = prob.model.list_outputs(values=False, out_stream=None, tags="tag_wrong")
+        self.assertEqual(sorted(outputs), [])
+
+    def test_feature_has_diag_partials(self):
         import numpy as np
         import openmdao.api as om
 
@@ -809,7 +910,7 @@ class TestExecComp(unittest.TestCase):
         model = p.model
         model.add_subsystem('indep', om.IndepVarComp('x', val=np.ones(5)))
 
-        model.add_subsystem('comp', om.ExecComp('y=3.0*x + 2.5', vectorize=True, x=np.ones(5), y=np.ones(5)))
+        model.add_subsystem('comp', om.ExecComp('y=3.0*x + 2.5', has_diag_partials=True, x=np.ones(5), y=np.ones(5)))
         model.connect('indep.x', 'comp.x')
 
         p.setup()
@@ -965,7 +1066,7 @@ class TestExecComp(unittest.TestCase):
 class TestExecCompParameterized(unittest.TestCase):
 
     @parameterized.expand(itertools.product([
-      func_name for func_name in _expr_dict if not func_name.startswith('_')
+      func_name for func_name in _expr_dict if func_name not in ('np', 'numpy', '__builtins__')
     ]), name_func=lambda f, n, p: 'test_exec_comp_value_' + '_'.join(a for a in p.args))
     def test_exec_comp_value(self, f):
         test_data = _ufunc_test_data[f]
@@ -1010,7 +1111,7 @@ class TestExecCompParameterized(unittest.TestCase):
                 print(f, 'does not support complex-step differentiation')
 
     @parameterized.expand(itertools.product([
-      func_name for func_name in _expr_dict if not func_name.startswith('_')
+      func_name for func_name in _expr_dict if func_name not in ('np', 'numpy', '__builtins__')
     ]), name_func=lambda f, n, p: 'test_exec_comp_jac_' + '_'.join(a for a in p.args))
     def test_exec_comp_jac(self, f):
         test_data = _ufunc_test_data[f]
