@@ -2,17 +2,14 @@
 Unit tests for the structured metamodel component.
 """
 from __future__ import division, print_function, absolute_import
-from copy import deepcopy
 import unittest
 from six import assertRaisesRegex
 
 import numpy as np
+from numpy.testing import assert_almost_equal
 
 import openmdao.api as om
-from openmdao.utils.assert_utils import assert_rel_error, assert_warning
-
-from numpy.testing import (assert_array_almost_equal, assert_almost_equal,
-                           assert_allclose, assert_array_equal, assert_equal)
+from openmdao.utils.assert_utils import assert_rel_error, assert_warning, assert_check_partials
 
 scipy_gte_019 = True
 try:
@@ -20,8 +17,6 @@ try:
 except ImportError:
     scipy_gte_019 = False
 
-if scipy_gte_019:
-    from openmdao.components.meta_model_structured_comp import _RegularGridInterp, OutOfBoundsError
 
 x = np.array([-0.97727788, -0.15135721, -0.10321885,  0.40015721,  0.4105985,
                0.95008842,  0.97873798,  1.76405235,  1.86755799,  2.2408932 ])
@@ -356,10 +351,6 @@ g = np.array([
 ])
 
 
-def rel_error(actual, computed):
-    return np.linalg.norm(actual - computed) / np.linalg.norm(actual)
-
-
 class SampleMap(object):
     param_data = []
     np.random.seed(0)
@@ -389,394 +380,7 @@ class SampleMap(object):
 
 
 @unittest.skipIf(not scipy_gte_019, "only run if scipy>=0.19.")
-class TestRegularGridInterpolator(unittest.TestCase):
-    """Tests the functionality of the regular grid interpolator."""
-
-    def setUp(self):
-        self.config = _RegularGridInterp._interp_methods()
-        self.valid_methods, self.interp_configs = self.config
-
-    def _get_sample_4d_large(self):
-        def f(x, y, z, w):
-            return x**2 + y**2 + z**2 + w**2
-        X = np.linspace(-10, 10, 6)
-        Y = np.linspace(-10, 10, 7)
-        np.random.seed(0)
-        Z = np.random.uniform(-10, 10, 6)
-        Z.sort()
-        W = np.linspace(-10, 10, 8)
-        points = [X, Y, Z, W]
-        values = f(*np.meshgrid(*points, indexing='ij'))
-        return points, values
-
-    def _get_sample_2d(self):
-        # test problem with enough points for smooth spline fits
-        def f(u, v):
-            return u * np.cos(u * v) + v * np.sin(u * v)
-
-        def df(u, v):
-            return (-u * v * np.sin(u * v) + v**2 * np.cos(u * v) +
-                    np.cos(u * v),
-                    -u**2 * np.sin(u * v) + u * v * np.cos(u * v) +
-                    np.sin(u * v))
-
-        # uniformly spaced axis
-        u = np.linspace(0, 3, 50)
-        # randomly spaced axis
-        np.random.seed(7590)
-        v = np.random.uniform(0, 3, 50)
-        v.sort()
-
-        points = [u, v]
-        values = f(*np.meshgrid(*points, indexing='ij'))
-        return points, values, f, df
-
-    def test_list_input(self):
-        points, values = self._get_sample_4d_large()
-
-        sample = np.asarray([[0.1, 0.1, 1., .9], [0.2, 0.1, .45, .8],
-                             [0.5, 0.5, .5, .5]])
-
-        for method in self.valid_methods:
-            interp = _RegularGridInterp(points, values.tolist(), method=method)
-            v1 = interp(sample.tolist(), compute_gradients=False)
-
-            interp = _RegularGridInterp(points, values, method=method)
-            v2 = interp(sample, compute_gradients=False)
-
-            assert_allclose(v1, v2)
-
-    def test_auto_reduce_spline_order(self):
-        # if a spline method is used and spline_dim_error=False and a dimension
-        # does not have enough points, the spline order for that dimension
-        # should be automatically reduced
-        np.random.seed(314)
-
-        # x dimension is too small for cubic, should fall back to linear
-        x = [0, 1]
-        y = np.linspace(-10, 4, 10)
-        z = np.linspace(1000, 2000, 20)
-
-        points = [x, y, z]
-        values = np.random.randn(2, 10, 20)
-
-        # verify that this raises error with dimension checking
-        self.assertRaises(ValueError, _RegularGridInterp,
-                          points, values, 'cubic')
-
-        interp = _RegularGridInterp(
-            points, values, method='cubic', spline_dim_error=False)
-
-        # first dimension (x) should be reduced to k=1 (linear)
-        assert_equal(interp._ki[0], 1)
-
-        # should operate as normal
-        x = [0.5, 0, 1001]
-        result = interp(x)
-        assert_almost_equal(result, -0.046325695741704434, decimal=5)
-
-        interp = _RegularGridInterp(
-            points, values, method='slinear', spline_dim_error=False)
-
-        value1 = interp(x)
-        # cycle through different methods that require order reduction
-        # in the first dimension
-        value2 = interp(x, method='quintic')
-        interp.gradient(x, method='quintic')
-        value3 = interp(x, method='cubic')
-        interp.gradient(x, method='cubic')
-        # use default method again
-        value4 = interp(x)
-
-        # values from different methods should be different
-        self.assertRaises(AssertionError, assert_equal, value1, value2)
-        self.assertRaises(AssertionError, assert_equal, value2, value3)
-
-        # first value should match last with no side effects from the
-        # order reduction or gradient caluclations
-        assert_equal(value1, value4)
-
-    def test_complex_exception_spline(self):
-        points, values = self._get_sample_4d_large()
-        values = values - 2j * values
-        sample = np.asarray([[0.1, 0.1, 1., .9]])
-
-        # spline methods dont support complex values
-        for method in self.valid_methods:
-            self.assertRaises(ValueError, _RegularGridInterp, points, values,
-                              method)
-
-    def test_minimum_required_gridsize(self):
-        for method in self.valid_methods:
-            k = self.interp_configs[method]
-            x = np.linspace(0, 1, k)
-            y = np.linspace(0, 1, k)
-            points = [x, y]
-            X, Y = np.meshgrid(*points, indexing='ij')
-            values = X + Y
-            self.assertRaises(ValueError, _RegularGridInterp, points, values,
-                              method)
-
-    def test_method_switching(self):
-        # should be able to switch interpolation methods on each __call__
-        # and gradient call, without overriding defaults permenantly.
-        # exceptions and gradient caching should work as expected.
-
-        np.random.seed(314)
-        x = np.linspace(-100, 2, 10)
-        y = np.linspace(-10, 4, 6)
-        z = np.linspace(1000, 2000, 50)
-
-        points = [x, y, z]
-        values = np.random.randn(10, 6, 50)
-
-        x = [0.5, 0, 1001]
-
-        # create as cubic
-        interp = _RegularGridInterp(
-            points, values, method='cubic')
-
-        # value and gradient work as expected
-        result1 = interp(x)
-        gradient1 = interp.gradient(x)
-        result_actual_1 = 0.2630309995970872
-        result_gradient_1 = np.array([0.22505535, -0.46465198, 0.02523666])
-
-        assert_almost_equal(result1, result_actual_1)
-        assert_almost_equal(gradient1, result_gradient_1)
-
-        # changing the method should work as expected
-        result2 = interp(x, method='slinear')
-        gradient2 = interp.gradient(x, method='slinear')
-        result_actual_2 = 0.27801704674026684
-        result_gradient_2 = np.array([0.12167214, -0.44221416, -0.00323078])
-
-        assert_almost_equal(result2, result_actual_2)
-        assert_almost_equal(gradient2, result_gradient_2)
-
-        # should be able to switch back and get the original results without
-        # explicitly setting the method
-        result3 = interp(x)
-        gradient3 = interp.gradient(x)
-        assert_almost_equal(result3, result_actual_1)
-        assert_almost_equal(gradient3, result_gradient_1)
-
-        # new interpolator and evaluation point
-        interp = _RegularGridInterp(
-            points, values, method='slinear')
-        # values will be cast to float for splines/gradient methods
-        # otherwise, will get null vector gradient [0,0,0] at all pts
-        x = [-50, 0, 1501]
-        result6 = interp(x)
-        result_actual_6 = 0.3591176338294626
-        assert_almost_equal(result6, result_actual_6)
-
-        # should be able to switch and get value and gradient
-        result7 = interp(x, method='quintic')
-        gradient7 = interp.gradient(x, method='quintic')
-        result_actual_7 = 0.6157594079479937
-        result_gradient_7 = np.array([-0.35731922, 0.23131539, -0.14088582])
-        assert_almost_equal(result7, result_actual_7)
-        assert_almost_equal(gradient7, result_gradient_7)
-
-        # switch again; gradient should be different
-        gradient8 = interp.gradient(x, method='slinear')
-        result_gradient_8 = np.array([-0.11299396, 0.24352342, -0.07446338])
-        assert_almost_equal(gradient8, result_gradient_8)
-
-        # should be able to switch back to original without setting it
-        result9 = interp(x)
-        assert_almost_equal(result9, result6)
-
-    def test_spline_deriv_xi1d(self):
-        # tests gradient values
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(1234)
-        test_pt = np.random.uniform(0, 3, 2)
-        actual = np.array(df(*test_pt))
-        tol = 1e-1
-        for method in self.valid_methods:
-            if method == 'slinear':
-                tol = 1.5
-            interp = _RegularGridInterp(points, values, method)
-            computed = interp.gradient(test_pt)
-            r_err = rel_error(actual, computed)
-            assert r_err < tol
-
-            # test that gradients have been cached
-            assert_array_equal(interp._xi, test_pt)
-            assert_array_equal(
-                interp._all_gradients.flatten(), computed.flatten())
-
-    def test_gradients_returned_by_xi(self):
-        # verifies that gradients with respect to xi are returned if cached
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(4321)
-        for method in self.valid_methods:
-            interp = _RegularGridInterp(points, values, method)
-            x = np.array([0.9, 0.1])
-            interp._xi = x
-            interp._gmethod = method
-            dy = np.array([0.997901, 0.08915])
-            interp._all_gradients = dy
-            assert_almost_equal(interp.gradient(x), dy)
-
-    def test_spline_xi1d(self):
-        # test interpolated values
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(1)
-        test_pt = np.random.uniform(0, 3, 2)
-        actual = func(*test_pt)
-        tol = 1e-2
-        for method in self.valid_methods:
-            if method == 'slinear':
-                tol = 0.5
-            interp = _RegularGridInterp(points, values, method)
-            computed = interp(test_pt, compute_gradients=False)
-            r_err = rel_error(actual, computed)
-            assert r_err < tol
-
-    def test_spline_out_of_bounds_extrap(self):
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(5)
-        test_pt = np.random.uniform(3, 3.1, 2)
-        actual = func(*test_pt)
-        gradient = np.array(df(*test_pt))
-        tol = 1e-1
-        for method in self.valid_methods:
-            k = self.interp_configs[method]
-            if method == 'slinear':
-                tol = 2
-            interp = _RegularGridInterp(points, values, method,
-                                        bounds_error=False,
-                                        fill_value=None)
-            computed = interp(test_pt)
-            computed_grad = interp.gradient(test_pt)
-            r_err = rel_error(actual, computed)
-            assert r_err < tol
-
-            r_err = rel_error(gradient, computed_grad)
-            # extrapolated gradients are even trickier, but usable still
-            assert r_err < 2 * tol
-
-    def test_spline_xi3d(self):
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(1)
-        test_pt = np.random.uniform(0, 3, 6).reshape(3, 2)
-        actual = func(*test_pt.T)
-        for method in self.valid_methods:
-            tol = 1e-1
-            if method == 'slinear':
-                tol = 0.5
-            interp = _RegularGridInterp(points, values, method)
-            computed = interp(test_pt, compute_gradients=False)
-            r_err = rel_error(actual, computed)
-            assert r_err < tol
-
-    def test_out_of_bounds_fill2(self):
-        points, values, func, df = self. _get_sample_2d()
-        np.random.seed(1)
-        test_pt = np.random.uniform(3, 3.1, 2)
-        actual = np.asarray([np.nan])
-        methods = self.valid_methods
-        for method in methods:
-            interp = _RegularGridInterp(points, values, method,
-                                        bounds_error=False,
-                                        fill_value=np.nan)
-            computed = interp(test_pt, compute_gradients=False)
-            assert_array_almost_equal(computed, actual)
-
-    def test_invalid_fill_value(self):
-        np.random.seed(1234)
-        x = np.linspace(0, 2, 5)
-        y = np.linspace(0, 1, 7)
-        values = np.random.rand(5, 7)
-
-        # integers can be cast to floats
-        _RegularGridInterp((x, y), values, fill_value=1)
-
-        # complex values cannot
-        self.assertRaises(ValueError, _RegularGridInterp,
-                          (x, y), values, fill_value=1 + 2j)
-
-    def test_NaN_exception(self):
-        np.random.seed(1234)
-        x = np.linspace(0, 2, 5)
-        y = np.linspace(0, 1, 7)
-        values = np.random.rand(5, 7)
-        interp = _RegularGridInterp((x, y), values)
-
-        with self.assertRaises(OutOfBoundsError) as cm:
-            interp([1, np.nan])
-
-        err = cm.exception
-
-        self.assertEqual(str(err), 'One of the requested xi contains a NaN')
-        self.assertEqual(err.idx, 1)
-        self.assertTrue(np.isnan(err.value))
-        self.assertEqual(err.lower, 0)
-        self.assertEqual(err.upper, 1)
-
-    def test_error_messages(self):
-        # For coverage. Most of these errors are probably not reachable in openmdao, but
-        # proper unit testing requires them for standalone usage of the Interpolation.
-        points, values = self._get_sample_4d_large()
-
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(points, values.tolist(), method='junk')
-
-        msg = ('Method "junk" is not defined. Valid methods are')
-        self.assertTrue(str(cm.exception).startswith(msg))
-
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(points, values.tolist()[1])
-
-        msg = ('There are 4 point arrays, but values has 3 dimensions')
-        self.assertEqual(str(cm.exception), msg)
-
-        badpoints = deepcopy(points)
-        badpoints[0][0] = 55.0
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(badpoints, values.tolist())
-
-        msg = ('The points in dimension 0 must be strictly ascending')
-        self.assertEqual(str(cm.exception), msg)
-
-        badpoints[0] = np.vstack((np.arange(6), np.arange(6)))
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(badpoints, values.tolist())
-
-        msg = ('The points in dimension 0 must be 1-dimensional')
-        self.assertEqual(str(cm.exception), msg)
-
-        badpoints[0] = (np.arange(4))
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(badpoints, values.tolist())
-
-        msg = ('There are 4 points and 6 values in dimension 0')
-        self.assertEqual(str(cm.exception), msg)
-
-        badvalues = np.array(values, dtype=np.complex)
-        with self.assertRaises(ValueError) as cm:
-            interp = _RegularGridInterp(badpoints, badvalues.tolist())
-
-        msg = ("method 'slinear' does not support complex values.")
-        self.assertEqual(str(cm.exception), msg)
-
-        interp = _RegularGridInterp(points, values.tolist())
-        x = [0.5, 0, 0.5, 0.9]
-
-        with self.assertRaises(ValueError) as cm:
-            computed = interp(x, method='junk')
-
-        msg = ('Method "junk" is not defined. Valid methods are')
-        self.assertTrue(str(cm.exception).startswith(msg))
-
-        self.assertEqual(set(interp.methods()), set(["quintic", "cubic", "slinear"]))
-
-
-@unittest.skipIf(not scipy_gte_019, "only run if scipy>=0.19.")
-class TestRegularGridMap(unittest.TestCase):
+class TestMetaModelStructuredScipy(unittest.TestCase):
     """
     Tests the regular grid map component. specifically the analytic derivatives
     vs. finite difference estimates.
@@ -799,7 +403,7 @@ class TestRegularGridMap(unittest.TestCase):
 
         model.add_subsystem('des_vars', ivc, promotes=["*"])
 
-        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=True)
+        comp = om.MetaModelStructuredComp(method='scipy_slinear', extrapolate=True)
 
         for param in params:
             comp.add_input(param['name'], param['default'], param['values'])
@@ -843,7 +447,7 @@ class TestRegularGridMap(unittest.TestCase):
 
         model.add_subsystem('des_vars', ivc, promotes=["*"])
 
-        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=True)
+        comp = om.MetaModelStructuredComp(method='scipy_slinear', extrapolate=True)
 
         for out in outs:
             comp.add_output(out['name'], out['default'], out['values'])
@@ -874,24 +478,12 @@ class TestRegularGridMap(unittest.TestCase):
         self.run_and_check_derivs(self.prob)
 
     def test_deriv4(self):
+        # Tests extrapolation.
         self.prob['x'] = 65.0
         self.prob['y'] = 0.951
         self.prob['z'] = 2.5
         self.run_and_check_derivs(self.prob)
 
-    def test_raise_interp_error(self):
-        # muck with the grid to trigger an error in the interp call
-        self.prob.model.comp.interps['f'].grid = []
-
-        # verify that the error is raised by the meta model comp with
-        # information to help locate the error
-        with self.assertRaises(Exception) as context:
-            self.run_and_check_derivs(self.prob)
-        self.assertEqual(str(context.exception),
-                         "Error interpolating output 'f' in comp:\n"
-                         "The requested sample points xi have dimension 3, "
-                         "but this RegularGridInterp has dimension 0")
-
     def test_raise_out_of_bounds_error(self):
         model = om.Group()
         ivc = om.IndepVarComp()
@@ -909,7 +501,7 @@ class TestRegularGridMap(unittest.TestCase):
         model.add_subsystem('des_vars', ivc, promotes=["*"])
 
         # Need to make sure extrapolate is False for bounds to be checked
-        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=False)
+        comp = om.MetaModelStructuredComp(method='scipy_slinear', extrapolate=False)
 
         for param in params:
             comp.add_input(param['name'], param['default'], param['values'])
@@ -928,7 +520,7 @@ class TestRegularGridMap(unittest.TestCase):
         # The interpolating output name is given as a regexp because the exception could
         #   happen with f or g first. The order those are evaluated comes from the keys of
         #   dict so no guarantee on the order except for Python 3.6 !
-        msg = "Error interpolating output '[f|g]' in 'comp' because input 'comp.z' was " \
+        msg = "MetaModelStructuredComp \(comp\): Error interpolating output '[f|g]' because input 'comp.z' was " \
               "out of bounds \('.*', '.*'\) with value '9.0'"
         with assertRaisesRegex(self, om.AnalysisError, msg):
             self.run_and_check_derivs(self.prob)
@@ -950,7 +542,7 @@ class TestRegularGridMap(unittest.TestCase):
         ivc.add_output('g_train', outs[1]['values'])
 
         comp = om.MetaModelStructuredComp(training_data_gradients=True,
-                                          method='cubic',
+                                          method='scipy_cubic',
                                           vec_size=3)
         for param in params:
             comp.add_input(param['name'], param['default'], param['values'])
@@ -993,7 +585,7 @@ class TestRegularGridMap(unittest.TestCase):
         ivc.add_output('g_train', outs[1]['values'])
 
         comp = om.MetaModelStructuredComp(training_data_gradients=True,
-                                          method='cubic',
+                                          method='scipy_cubic',
                                           vec_size=3)
         for param in params:
             comp.add_input(param['name'], param['default'], param['values'])
@@ -1006,7 +598,6 @@ class TestRegularGridMap(unittest.TestCase):
                             comp,
                             promotes=["*"])
 
-
         prob = om.Problem(model)
         prob.setup()
         prob.run_model()
@@ -1072,12 +663,379 @@ class TestRegularGridMap(unittest.TestCase):
         with self.assertRaises(om.AnalysisError) as cm:
             p.run_model()
 
-        msg = ("Error interpolating output 'y' in 'MM' because input 'MM.x' was out of bounds ('0.0', '1.0') with value '1.1'")
+        msg = ("MMComp (MM): Error interpolating output 'y' because input 'MM.x' was out of bounds ('0.0', '1.0') with value '1.1'")
         self.assertEqual(str(cm.exception), msg)
 
 
+class TestMetaModelStructuredPython(unittest.TestCase):
+    """
+    Tests the regular grid map component. specifically the analytic derivatives
+    vs. finite difference estimates.
+    """
+
+    def setUp(self):
+
+        model = om.Group()
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, z = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', x['default'], units=x['units'])
+        ivc.add_output('y', y['default'], units=y['units'])
+        ivc.add_output('z', z['default'], units=z['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=True)
+
+        for param in params:
+            comp.add_input(param['name'], param['default'], param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], out['default'], out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+        self.prob = om.Problem(model)
+        self.prob.setup()
+        self.prob['x'] = 1.0
+        self.prob['y'] = 0.75
+        self.prob['z'] = -1.7
+
+    def run_and_check_derivs(self, prob, tol=1e-5, verbose=False):
+        """Runs check_partials and compares to analytic derivatives."""
+
+        prob.run_model()
+        derivs = prob.check_partials(out_stream=None)
+
+        for i in derivs['comp'].keys():
+            if verbose:
+                print("Checking derivative pair:", i)
+            if derivs['comp'][i]['J_fwd'].sum() != 0.0:
+                rel_err = max(derivs['comp'][i]['rel error'])
+                self.assertLessEqual(rel_err, tol)
+
+    def test_deriv1(self):
+        # run at default pt
+        self.run_and_check_derivs(self.prob)
+
+        # test output values
+        f, g = self.prob['comp.f'], self.prob['comp.g']
+
+        tol = 1e-6
+        assert_rel_error(self, f, -0.05624571, tol)
+        assert_rel_error(self, g, 1.02068754, tol)
+
+    def test_deriv1_swap(self):
+        # Bugfix test that we can add outputs before inputs.
+
+        model = om.Group()
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, z = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', x['default'], units=x['units'])
+        ivc.add_output('y', y['default'], units=y['units'])
+        ivc.add_output('z', z['default'], units=z['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=True)
+
+        for out in outs:
+            comp.add_output(out['name'], out['default'], out['values'])
+
+        for param in params:
+            comp.add_input(param['name'], param['default'], param['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+        prob = om.Problem(model)
+        prob.setup()
+        prob['x'] = 1.0
+        prob['y'] = 0.75
+        prob['z'] = -1.7
+
+        # run at default pt
+        self.run_and_check_derivs(prob)
+
+    def test_deriv2(self):
+        self.prob['x'] = 10.0
+        self.prob['y'] = 0.81
+        self.prob['z'] = 1.1
+        self.run_and_check_derivs(self.prob)
+
+    def test_deriv3(self):
+        self.prob['x'] = 90.0
+        self.prob['y'] = 1.2
+        self.prob['z'] = 2.1
+        self.run_and_check_derivs(self.prob)
+
+    def test_deriv4(self):
+        # Tests extrapolation.
+        self.prob['x'] = 65.0
+        self.prob['y'] = 0.951
+        self.prob['z'] = 2.5
+        self.run_and_check_derivs(self.prob)
+
+    def test_vectorized_linear(self):
+        prob = om.Problem()
+        model = prob.model
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, _ = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', np.array([x['default'], x['default'], x['default']]),
+                       units=x['units'])
+        ivc.add_output('y', np.array([y['default'], y['default'], y['default']]),
+                       units=x['units'])
+        ivc.add_output('z', np.array([z['default'], z['default'], z['default']]),
+                       units=x['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='slinear', extrapolate=True, vec_size=3)
+
+        for param in params:
+            comp.add_input(param['name'], np.array([param['default'], param['default'], param['default']]),
+                           param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], np.array([out['default'], out['default'], out['default']]),
+                            out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+
+        prob.setup(force_alloc_complex=True)
+        prob['x'] = np.array([1.0, 10.0, 90.0])
+        prob['y'] = np.array([0.75, 0.81, 1.2])
+        prob['z'] = np.array([-1.7, 1.1, 2.1])
+
+        prob.run_model()
+
+        partials = prob.check_partials(method='cs', out_stream=None)
+        assert_check_partials(partials, rtol=1e-10)
+
+    def test_vectorized_lagrange2(self):
+        prob = om.Problem()
+        model = prob.model
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, _ = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', np.array([x['default'], x['default'], x['default']]),
+                       units=x['units'])
+        ivc.add_output('y', np.array([y['default'], y['default'], y['default']]),
+                       units=x['units'])
+        ivc.add_output('z', np.array([z['default'], z['default'], z['default']]),
+                       units=x['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='lagrange2', extrapolate=True, vec_size=3)
+
+        for param in params:
+            comp.add_input(param['name'], np.array([param['default'], param['default'], param['default']]),
+                           param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], np.array([out['default'], out['default'], out['default']]),
+                            out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+
+        prob.setup(force_alloc_complex=True)
+        prob['x'] = np.array([1.0, 10.0, 90.0])
+        prob['y'] = np.array([0.75, 0.81, 1.2])
+        prob['z'] = np.array([-1.7, 1.1, 2.1])
+
+        prob.run_model()
+
+        partials = prob.check_partials(method='cs', out_stream=None)
+        # Derivs are large, so ignore atol.
+        assert_check_partials(partials, atol=1e10, rtol=1e-10)
+
+    def test_vectorized_lagrange3(self):
+        prob = om.Problem()
+        model = prob.model
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, _ = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', np.array([x['default'], x['default'], x['default']]),
+                       units=x['units'])
+        ivc.add_output('y', np.array([y['default'], y['default'], y['default']]),
+                       units=x['units'])
+        ivc.add_output('z', np.array([z['default'], z['default'], z['default']]),
+                       units=x['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='lagrange3', extrapolate=True, vec_size=3)
+
+        for param in params:
+            comp.add_input(param['name'], np.array([param['default'], param['default'], param['default']]),
+                           param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], np.array([out['default'], out['default'], out['default']]),
+                            out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+
+        prob.setup(force_alloc_complex=True)
+        prob['x'] = np.array([1.0, 10.0, 90.0])
+        prob['y'] = np.array([0.75, 0.81, 1.2])
+        prob['z'] = np.array([-1.7, 1.1, 2.1])
+
+        prob.run_model()
+
+        partials = prob.check_partials(method='cs', out_stream=None)
+        # Derivs are large, so ignore atol.
+        assert_check_partials(partials, atol=1e10, rtol=1e-10)
+
+    def test_vectorized_akima(self):
+        prob = om.Problem()
+        model = prob.model
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, _ = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', np.array([x['default'], x['default'], x['default']]),
+                       units=x['units'])
+        ivc.add_output('y', np.array([y['default'], y['default'], y['default']]),
+                       units=x['units'])
+        ivc.add_output('z', np.array([z['default'], z['default'], z['default']]),
+                       units=x['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='akima', extrapolate=True, vec_size=3)
+
+        for param in params:
+            comp.add_input(param['name'], np.array([param['default'], param['default'], param['default']]),
+                           param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], np.array([out['default'], out['default'], out['default']]),
+                            out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+
+        prob.setup(force_alloc_complex=True)
+        prob['x'] = np.array([1.0, 10.0, 90.0])
+        prob['y'] = np.array([0.75, 0.81, 1.2])
+        prob['z'] = np.array([-1.7, 1.1, 2.1])
+
+        prob.run_model()
+
+        partials = prob.check_partials(method='cs', out_stream=None)
+        # Derivs are large, so ignore atol.
+        assert_check_partials(partials, atol=1e10, rtol=1e-10)
+
+    def test_vectorized_cubic(self):
+        prob = om.Problem()
+        model = prob.model
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        x, y, _ = params
+        outs = mapdata.output_data
+        z = outs[0]
+        ivc.add_output('x', np.array([x['default'], x['default'], x['default']]),
+                       units=x['units'])
+        ivc.add_output('y', np.array([y['default'], y['default'], y['default']]),
+                       units=x['units'])
+        ivc.add_output('z', np.array([z['default'], z['default'], z['default']]),
+                       units=x['units'])
+
+        model.add_subsystem('des_vars', ivc, promotes=["*"])
+
+        comp = om.MetaModelStructuredComp(method='cubic', extrapolate=True, vec_size=3)
+
+        for param in params:
+            comp.add_input(param['name'], np.array([param['default'], param['default'], param['default']]),
+                           param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], np.array([out['default'], out['default'], out['default']]),
+                            out['values'])
+
+        model.add_subsystem('comp', comp, promotes=["*"])
+
+        prob.setup(force_alloc_complex=True)
+        prob['x'] = np.array([1.0, 10.0, 90.0])
+        prob['y'] = np.array([0.75, 0.81, 1.2])
+        prob['z'] = np.array([-1.7, 1.1, 2.1])
+
+        prob.run_model()
+
+        partials = prob.check_partials(method='cs', out_stream=None)
+        # Derivs are large, so ignore atol.
+        assert_check_partials(partials, atol=1e10, rtol=1e-10)
+
+    def test_training_gradient(self):
+        model = om.Group()
+        ivc = om.IndepVarComp()
+
+        mapdata = SampleMap()
+
+        params = mapdata.param_data
+        outs = mapdata.output_data
+
+        ivc.add_output('x', np.array([-0.3, 0.7, 1.2]))
+        ivc.add_output('y', np.array([0.14, 0.313, 1.41]))
+        ivc.add_output('z', np.array([-2.11, -1.2, 2.01]))
+
+        ivc.add_output('f_train', outs[0]['values'])
+        ivc.add_output('g_train', outs[1]['values'])
+
+        comp = om.MetaModelStructuredComp(training_data_gradients=True,
+                                          method='lagrange3', vec_size=3)
+        for param in params:
+            comp.add_input(param['name'], param['default'], param['values'])
+
+        for out in outs:
+            comp.add_output(out['name'], out['default'], out['values'])
+
+        model.add_subsystem('ivc', ivc, promotes=["*"])
+        model.add_subsystem('comp',
+                            comp,
+                            promotes=["*"])
+
+        prob = om.Problem(model)
+        prob.setup()
+        prob.run_model()
+
+        self.run_and_check_derivs(prob)
+
+
 @unittest.skipIf(not scipy_gte_019, "only run if scipy>=0.19.")
-class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
+class TestMetaModelStructuredCompFeature(unittest.TestCase):
 
     @unittest.skipIf(not scipy_gte_019, "only run if scipy>=0.19.")
     def test_xor(self):
@@ -1085,7 +1043,7 @@ class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
         import openmdao.api as om
 
         # Create regular grid interpolator instance
-        xor_interp = om.MetaModelStructuredComp(method='slinear')
+        xor_interp = om.MetaModelStructuredComp(method='scipy_slinear')
 
         # set up inputs and outputs
         xor_interp.add_input('x', 0.0, training_data=np.array([0.0, 1.0]), units=None)
@@ -1135,7 +1093,7 @@ class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
         print(f.shape)
 
         # Create regular grid interpolator instance
-        interp = om.MetaModelStructuredComp(method='cubic')
+        interp = om.MetaModelStructuredComp(method='scipy_cubic')
         interp.add_input('p1', 0.5, training_data=p1)
         interp.add_input('p2', 0.0, training_data=p2)
         interp.add_input('p3', 3.14, training_data=p3)
@@ -1178,7 +1136,7 @@ class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
         f = np.sqrt(P1) + P2 * P3
 
         # Create regular grid interpolator instance
-        interp = om.MetaModelStructuredComp(method='cubic', vec_size=2)
+        interp = om.MetaModelStructuredComp(method='scipy_cubic', vec_size=2)
         interp.add_input('p1', 0.5, training_data=p1)
         interp.add_input('p2', 0.0, training_data=p2)
         interp.add_input('p3', 3.14, training_data=p3)
@@ -1221,7 +1179,7 @@ class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
         print(f.shape)
 
         # Create regular grid interpolator instance
-        interp = om.MetaModelStructuredComp(method='cubic', training_data_gradients=True)
+        interp = om.MetaModelStructuredComp(method='scipy_cubic', training_data_gradients=True)
         interp.add_input('p1', 0.5, p1)
         interp.add_input('p2', 0.0, p2)
         interp.add_input('p3', 3.14, p3)
@@ -1258,7 +1216,7 @@ class TestMetaModelStructuredCompMapFeature(unittest.TestCase):
         msg = "'MetaModelStructured' has been deprecated. Use 'MetaModelStructuredComp' instead."
 
         with assert_warning(DeprecationWarning, msg):
-            xor_interp = om.MetaModelStructured(method='slinear')
+            xor_interp = om.MetaModelStructured(method='scipy_slinear')
 
         # set up inputs and outputs
         xor_interp.add_input('x', 0.0, training_data=np.array([0.0, 1.0]), units=None)
