@@ -112,7 +112,7 @@ class Driver(object):
         self._responses = None
 
         # Driver options
-        self.options = OptionsDictionary()
+        self.options = OptionsDictionary(parent_name=type(self).__name__)
 
         self.options.declare('debug_print', types=list, check_valid=_check_debug_print_opts_valid,
                              desc="List of what type of Driver variables to print at each "
@@ -121,7 +121,7 @@ class Driver(object):
                              default=[])
 
         # Case recording options
-        self.recording_options = OptionsDictionary()
+        self.recording_options = OptionsDictionary(parent_name=type(self).__name__)
 
         self.recording_options.declare('record_model_metadata', types=bool, default=True,
                                        desc='Record metadata for all Systems in the model')
@@ -147,7 +147,7 @@ class Driver(object):
                                        desc='Set to True to record inputs at the driver level')
 
         # What the driver supports.
-        self.supports = OptionsDictionary()
+        self.supports = OptionsDictionary(parent_name=type(self).__name__)
         self.supports.declare('inequality_constraints', types=bool, default=False)
         self.supports.declare('equality_constraints', types=bool, default=False)
         self.supports.declare('linear_constraints', types=bool, default=False)
@@ -162,14 +162,11 @@ class Driver(object):
         self.iter_count = 0
         self.cite = ""
 
-        self._coloring_info = {
-            'coloring': None,
-            'show_summary': True,
-            'show_sparsity': False,
-            'tol': 1e-15,
-            'orders': 20,
-            'num_full_jacs': 3,
-        }
+        self._coloring_info = coloring_mod._DEF_COMP_SPARSITY_ARGS.copy()
+        self._coloring_info['coloring'] = None
+        self._coloring_info['dynamic'] = False
+        self._coloring_info['static'] = None
+
         self._total_jac_sparsity = None
         self._res_jacs = {}
         self._total_jac = None
@@ -179,6 +176,18 @@ class Driver(object):
         self._declare_options()
         self.options.update(kwargs)
 
+    @property
+    def msginfo(self):
+        """
+        Return info to prepend to messages.
+
+        Returns
+        -------
+        str
+            Info to prepend to messages.
+        """
+        return type(self).__name__
+
     def add_recorder(self, recorder):
         """
         Add a recorder to the driver.
@@ -290,6 +299,10 @@ class Driver(object):
 
         # set up simultaneous deriv coloring
         if coloring_mod._use_total_sparsity:
+            # reset the coloring
+            if self._coloring_info['dynamic'] or self._coloring_info['static'] is not None:
+                self._coloring_info['coloring'] = None
+
             coloring = self._get_static_coloring()
             if coloring is not None and self.supports['simultaneous_derivatives']:
                 if model._owns_approx_jac:
@@ -996,6 +1009,7 @@ class Driver(object):
                          tol=coloring_mod._DEF_COMP_SPARSITY_ARGS['tol'],
                          orders=coloring_mod._DEF_COMP_SPARSITY_ARGS['orders'],
                          perturb_size=coloring_mod._DEF_COMP_SPARSITY_ARGS['perturb_size'],
+                         min_improve_pct=coloring_mod._DEF_COMP_SPARSITY_ARGS['min_improve_pct'],
                          show_summary=coloring_mod._DEF_COMP_SPARSITY_ARGS['show_summary'],
                          show_sparsity=coloring_mod._DEF_COMP_SPARSITY_ARGS['show_sparsity']):
         """
@@ -1011,6 +1025,9 @@ class Driver(object):
             Number of orders above and below the tolerance to check during the tolerance sweep.
         perturb_size : float
             Size of input/output perturbation during generation of sparsity.
+        min_improve_pct : float
+            If coloring does not improve (decrease) the number of solves more than the given
+            percentage, coloring will not be used.
         show_summary : bool
             If True, display summary information after generating coloring.
         show_sparsity : bool
@@ -1020,7 +1037,12 @@ class Driver(object):
         self._coloring_info['tol'] = tol
         self._coloring_info['orders'] = orders
         self._coloring_info['perturb_size'] = perturb_size
-        self._coloring_info['coloring'] = coloring_mod._DYN_COLORING
+        self._coloring_info['min_improve_pct'] = min_improve_pct
+        if self._coloring_info['static'] is None:
+            self._coloring_info['dynamic'] = True
+        else:
+            self._coloring_info['dynamic'] = False
+        self._coloring_info['coloring'] = None
         self._coloring_info['show_summary'] = show_summary
         self._coloring_info['show_sparsity'] = show_sparsity
 
@@ -1038,8 +1060,13 @@ class Driver(object):
         if self.supports['simultaneous_derivatives']:
             if coloring_mod._force_dyn_coloring and coloring is coloring_mod._STD_COLORING_FNAME:
                 # force the generation of a dynamic coloring this time
-                coloring = coloring_mod._DYN_COLORING
-            self._coloring_info['coloring'] = coloring
+                self._coloring_info['dynamic'] = True
+                self._coloring_info['static'] = None
+            else:
+                self._coloring_info['static'] = coloring
+                self._coloring_info['dynamic'] = False
+
+            self._coloring_info['coloring'] = None
         else:
             raise RuntimeError("Driver '%s' does not support simultaneous derivatives." %
                                self._get_name())
@@ -1080,16 +1107,22 @@ class Driver(object):
             The pre-existing or loaded Coloring, or None
         """
         info = self._coloring_info
-        coloring = info['coloring']
+        static = info['static']
 
-        if isinstance(coloring, coloring_mod.Coloring):
+        if isinstance(static, coloring_mod.Coloring):
+            coloring = static
+            info['coloring'] = coloring
+        else:
+            coloring = info['coloring']
+
+        if coloring is not None:
             return coloring
 
-        if coloring is coloring_mod._STD_COLORING_FNAME or isinstance(coloring, string_types):
-            if coloring is coloring_mod._STD_COLORING_FNAME:
+        if static is coloring_mod._STD_COLORING_FNAME or isinstance(static, string_types):
+            if static is coloring_mod._STD_COLORING_FNAME:
                 fname = self._get_total_coloring_fname()
             else:
-                fname = coloring
+                fname = static
             print("loading total coloring from file %s" % fname)
             coloring = info['coloring'] = coloring_mod.Coloring.load(fname)
             info.update(coloring._meta)
