@@ -157,7 +157,7 @@ class ExplicitComponent(Component):
             self._set_approx_partials_meta()
 
     def add_output(self, name, val=1.0, shape=None, units=None, res_units=None, desc='',
-                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None):
+                   lower=None, upper=None, ref=1.0, ref0=0.0, res_ref=None, tags=None):
         """
         Add an output variable to the component.
 
@@ -200,6 +200,9 @@ class ExplicitComponent(Component):
             Scaling parameter. The value in the user-defined res_units of this output's residual
             when the scaled value is 1. Default is None, which means residual scaling matches
             output scaling.
+        tags : str or list of strs
+            User defined tags that can be used to filter what gets listed when calling
+            list_inputs and list_outputs and also when listing results from case recorders.
 
         Returns
         -------
@@ -213,7 +216,8 @@ class ExplicitComponent(Component):
                                                          val=val, shape=shape, units=units,
                                                          res_units=res_units, desc=desc,
                                                          lower=lower, upper=upper,
-                                                         ref=ref, ref0=ref0, res_ref=res_ref)
+                                                         ref=ref, ref0=ref0, res_ref=res_ref,
+                                                         tags=tags)
 
     def _approx_subjac_keys_iter(self):
         for abs_key, meta in iteritems(self._subjacs_info):
