@@ -817,7 +817,7 @@ class Group(System):
         conns : dict
             Dictionary of connections passed down from parent group.
         """
-        global_abs_in2out = self._conn_global_abs_in2out
+        global_abs_in2out = self._conn_global_abs_in2out = {}
 
         allprocs_prom2abs_list_in = self._var_allprocs_prom2abs_list['input']
         allprocs_prom2abs_list_out = self._var_allprocs_prom2abs_list['output']
@@ -2079,10 +2079,7 @@ class Group(System):
         approx = self._get_approx_scheme(method)
         # reset the approx if necessary
         approx._exec_dict = defaultdict(list)
-        approx._approx_groups = None
-        if wrt_matches:
-            # if coloring is active, force regen of coloring approxs
-            approx._colored_approx_groups = None
+        approx._reset()
 
         approx_keys = self._get_approx_subjac_keys()
         for key in approx_keys:
