@@ -3,6 +3,7 @@
 from __future__ import division, print_function
 
 import sys
+import pprint
 import os
 import logging
 
@@ -39,7 +40,7 @@ from openmdao.utils.units import get_conversion
 from openmdao.utils import coloring as coloring_mod
 from openmdao.utils.name_maps import abs_key2rel_key
 from openmdao.vectors.default_vector import DefaultVector
-from openmdao.utils.logger_utils import get_logger
+from openmdao.utils.logger_utils import get_logger, TestLogger
 import openmdao.utils.coloring as coloring_mod
 
 try:
@@ -204,14 +205,14 @@ class Problem(object):
         self._rec_mgr = RecordingManager()
 
         # General options
-        self.options = OptionsDictionary()
+        self.options = OptionsDictionary(parent_name=type(self).__name__)
         self.options.declare('coloring_dir', types=str,
                              default=os.path.join(os.getcwd(), 'coloring_files'),
                              desc='Directory containing coloring files (if any) for this Problem.')
         self.options.update(options)
 
         # Case recording options
-        self.recording_options = OptionsDictionary()
+        self.recording_options = OptionsDictionary(parent_name=type(self).__name__)
 
         self.recording_options.declare('record_desvars', types=bool, default=True,
                                        desc='Set to True to record design variables at the '
@@ -661,6 +662,62 @@ class Problem(object):
         self.model._clear_iprint()
         return self.driver.run()
 
+    def compute_jacvec_product(self, of, wrt, mode, seed):
+        """
+        Given a seed and 'of' and 'wrt' variables, compute the total jacobian vector product.
+
+        Parameters
+        ----------
+        of : list of str
+            Variables whose derivatives will be computed.
+        wrt : list of str
+            Derivatives will be computed with respect to these variables.
+        mode : str
+            Derivative direction ('fwd' or 'rev').
+        seed : dict or list
+            Either a dict keyed by 'wrt' varnames (fwd) or 'of' varnames (rev), containing
+            dresidual (fwd) or doutput (rev) values, OR a list of dresidual or doutput
+            values that matches the corresponding 'wrt' (fwd) or 'of' (rev) varname list.
+
+        Returns
+        -------
+        dict
+            The total jacobian vector product, keyed by variable name.
+        """
+        if mode == 'fwd':
+            if len(wrt) != len(seed):
+                raise RuntimeError("seed and 'wrt' list must be the same length in fwd mode.")
+            lnames, rnames = of, wrt
+            lkind, rkind = 'output', 'residual'
+        else:  # rev
+            if len(of) != len(seed):
+                raise RuntimeError("seed and 'of' list must be the same length in rev mode.")
+            lnames, rnames = wrt, of
+            lkind, rkind = 'residual', 'output'
+
+        rvec = self.model._vectors[rkind]['linear']
+        lvec = self.model._vectors[lkind]['linear']
+
+        rvec._data[:] = 0.
+
+        # set seed values into dresids (fwd) or doutputs (rev)
+        try:
+            seed[rnames[0]]
+        except (IndexError, TypeError):
+            for i, name in enumerate(rnames):
+                rvec[name] = seed[i]
+        else:
+            for name in rnames:
+                rvec[name] = seed[name]
+
+        # We apply a -1 here because the derivative of the output is minus the derivative of
+        # the residual in openmdao.
+        rvec._data *= -1.
+
+        self.model.run_solve_linear(['linear'], mode)
+
+        return {n: lvec[n].copy() for n in lnames}
+
     def run_once(self):
         """
         Backward compatible call for run_model.
@@ -920,11 +977,12 @@ class Problem(object):
 
         driver._setup_driver(self)
 
-        coloring = driver._coloring_info['coloring']
-        if coloring is coloring_mod._STD_COLORING_FNAME:
+        info = driver._coloring_info
+        coloring = info['coloring']
+        if coloring is None and info['static'] is not None:
             coloring = driver._get_static_coloring()
-        if (coloring and coloring is not coloring_mod._DYN_COLORING and
-                coloring_mod._use_total_sparsity):
+
+        if coloring and coloring_mod._use_total_sparsity:
             # if we're using simultaneous total derivatives then our effective size is less
             # than the full size
             if coloring._fwd and coloring._rev:
@@ -950,13 +1008,18 @@ class Problem(object):
         # Now that setup has been called, we can set the iprints.
         for items in self._solver_print_cache:
             self.set_solver_print(level=items[0], depth=items[1], type_=items[2])
+        self._solver_print_cache = []
 
-        if self._check and self.comm.rank == 0:
+        if self._check:
             if self._check is True:
                 checks = _default_checks
             else:
                 checks = self._check
-            self.check_config(self._logger, checks=checks)
+            if self.comm.rank == 0:
+                logger = self._logger
+            else:
+                logger = TestLogger()
+            self.check_config(logger, checks=checks)
 
         if self._setup_status < 2:
             self._setup_status = 2
@@ -1279,7 +1342,7 @@ class Problem(object):
                     fd_options['form'] = form
                     fd_options['step_calc'] = step_calc
 
-                if step:
+                if step and requested_method == method:
                     fd_options['step'] = step
                 else:
                     fd_options['step'] = defaults['step']
@@ -1293,7 +1356,8 @@ class Problem(object):
 
                 all_fd_options[c_name][local_wrt] = fd_options
 
-                approximations[fd_options['method']].add_approximation(abs_key, fd_options)
+                approximations[fd_options['method']].add_approximation(abs_key, self.model,
+                                                                       fd_options)
 
             approx_jac = {}
             for approximation in itervalues(approximations):
@@ -1417,6 +1481,7 @@ class Problem(object):
         }
         approx = model._owns_approx_jac
         old_jac = model._jacobian
+        old_subjacs = model._subjacs_info.copy()
 
         model.approx_totals(method=method, step=step, form=form,
                             step_calc=step_calc if method is 'fd' else None)
@@ -1428,6 +1493,7 @@ class Problem(object):
         if not approx:
             model._jacobian = old_jac
             model._owns_approx_jac = False
+            model._subjacs_info = old_subjacs
 
         # Assemble and Return all metrics.
         data = {}
@@ -1459,7 +1525,7 @@ class Problem(object):
             Variables with respect to which the derivatives will be computed.
             Default is None, which uses the driver's desvars.
         return_format : string
-            Format to return the derivatives. Can be either 'dict' or 'flat_dict'.
+            Format to return the derivatives. Can be 'dict', 'flat_dict', or 'array'.
             Default is a 'flat_dict', which returns them in a dictionary whose keys are
             tuples of form (of, wrt).
         debug_print : bool
@@ -1663,7 +1729,7 @@ class Problem(object):
                 for col_name in have_array_values:
                     print("{}{}:".format((left_column_width + col_spacing) * ' ', col_name))
                     cell = row[col_name]
-                    out_str = str(cell)
+                    out_str = pprint.pformat(cell)
                     indented_lines = [(left_column_width + col_spacing) * ' ' +
                                       s for s in out_str.splitlines()]
                     print('\n'.join(indented_lines) + '\n')
@@ -1906,9 +1972,15 @@ def _assemble_derivative_data(derivative_data, rel_error_tol, abs_error_tol, out
                 else:
                     # If fd_norm is zero, let's use fwd_norm as the divisor for relative
                     # check. That way we don't accidentally squelch a legitimate problem.
-                    derivative_info['rel error'] = rel_err = ErrorTuple(fwd_error / fwd_norm,
-                                                                        rev_error / fwd_norm,
-                                                                        fwd_rev_error / fwd_norm)
+                    if totals:
+                        derivative_info['rel error'] = rel_err = ErrorTuple(fwd_error / fwd_norm,
+                                                                            nan,
+                                                                            nan)
+                    else:
+                        rel_err = ErrorTuple(fwd_error / fwd_norm,
+                                             rev_error / fwd_norm,
+                                             fwd_rev_error / fwd_norm)
+                        derivative_info['rel error'] = rel_err
 
             else:
                 if totals:
@@ -2146,3 +2218,29 @@ def _format_error(error, tol):
     if np.isnan(error) or error < tol:
         return '{:.6e}'.format(error)
     return '{:.6e} *'.format(error)
+
+
+class Slicer(object):
+    """
+    Helper class that can be used with the indices argument for Problem set_val and get_val.
+    """
+
+    def __getitem__(self, val):
+        """
+        Pass through indices or slice.
+
+        Parameters
+        ----------
+        val : int or slice object or tuples of slice objects
+            Indices or slice to return.
+
+        Returns
+        -------
+        indices : int or slice object or tuples of slice objects
+            Indices or slice to return.
+        """
+        return val
+
+
+# instance of the Slicer class to be used by users for the set_val and get_val methods of Problem
+slicer = Slicer()
