@@ -704,14 +704,14 @@ class TestGroupFiniteDifference(unittest.TestCase):
                                          -0.86236787, -0.97500023,  0.47739414,  0.51174103,  0.10052582]))
         indeps.add_output('r', .7)
 
-        arctan_yox = om.ExecComp('g=arctan(y/x)', vectorize=True,
+        arctan_yox = om.ExecComp('g=arctan(y/x)', has_diag_partials=True,
                                  g=np.ones(10), x=np.ones(10), y=np.ones(10))
 
         p.model.add_subsystem('arctan_yox', arctan_yox)
 
         p.model.add_subsystem('circle', om.ExecComp('area=pi*r**2'))
 
-        p.model.add_subsystem('r_con', om.ExecComp('g=x**2 + y**2 - r', vectorize=True,
+        p.model.add_subsystem('r_con', om.ExecComp('g=x**2 + y**2 - r', has_diag_partials=True,
                                                    g=np.ones(10), x=np.ones(10), y=np.ones(10)))
 
         p.model.connect('r', ('circle.r', 'r_con.r'))
@@ -756,25 +756,25 @@ class TestGroupFiniteDifference(unittest.TestCase):
                                          -0.86236787, -0.97500023,  0.47739414,  0.51174103,  0.10052582]))
         indeps.add_output('r', .7)
 
-        arctan_yox = om.ExecComp('g=arctan(y/x)', vectorize=True,
+        arctan_yox = om.ExecComp('g=arctan(y/x)', has_diag_partials=True,
                                  g=np.ones(10), x=np.ones(10), y=np.ones(10))
 
         p.model.add_subsystem('arctan_yox', arctan_yox)
 
         p.model.add_subsystem('circle', om.ExecComp('area=pi*r**2'))
 
-        p.model.add_subsystem('r_con', om.ExecComp('g=x**2 + y**2 - r', vectorize=True,
+        p.model.add_subsystem('r_con', om.ExecComp('g=x**2 + y**2 - r', has_diag_partials=True,
                                                    g=np.ones(10), x=np.ones(10), y=np.ones(10)))
 
         thetas = np.linspace(0, np.pi/4, 10)
-        p.model.add_subsystem('theta_con', om.ExecComp('g = x - theta', vectorize=True,
+        p.model.add_subsystem('theta_con', om.ExecComp('g = x - theta', has_diag_partials=True,
                                                        g=np.ones(10), x=np.ones(10),
                                                        theta=thetas))
-        p.model.add_subsystem('delta_theta_con', om.ExecComp('g = even - odd', vectorize=True,
+        p.model.add_subsystem('delta_theta_con', om.ExecComp('g = even - odd', has_diag_partials=True,
                                                              g=np.ones(10//2), even=np.ones(10//2),
                                                              odd=np.ones(10//2)))
 
-        p.model.add_subsystem('l_conx', om.ExecComp('g=x-1', vectorize=True, g=np.ones(10), x=np.ones(10)))
+        p.model.add_subsystem('l_conx', om.ExecComp('g=x-1', has_diag_partials=True, g=np.ones(10), x=np.ones(10)))
 
         IND = np.arange(10, dtype=int)
         ODD_IND = IND[1::2]  # all odd indices
@@ -856,8 +856,9 @@ class TestGroupCSMPI(unittest.TestCase):
             raise unittest.SkipTest("PETSc is not installed")
 
         prob = om.Problem()
+
         model = prob.model
-        sub = model.add_subsystem('sub', om.Group(), promotes=['*'])
+        sub = model.add_subsystem('sub', om.ParallelGroup(), promotes=['*'])
 
         model.add_subsystem('px', om.IndepVarComp('x', 1.0), promotes=['x'])
         model.add_subsystem('pz', om.IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
@@ -872,8 +873,9 @@ class TestGroupCSMPI(unittest.TestCase):
         model.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
         model.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
+
         sub.nonlinear_solver = om.NewtonSolver()
-        sub.linear_solver = om.DirectSolver(assemble_jac=False)
+        sub.linear_solver = om.DirectSolver()
         sub.nonlinear_solver.options['atol'] = 1e-10
         sub.nonlinear_solver.options['rtol'] = 1e-10
 
@@ -883,8 +885,8 @@ class TestGroupCSMPI(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        assert_rel_error(self, prob['y1'], 25.58830273, .00001)
-        assert_rel_error(self, prob['y2'], 12.05848819, .00001)
+        assert_rel_error(self, prob.get_val('y1', get_remote=True), 25.58830273, .00001)
+        assert_rel_error(self, prob.get_val('y2', get_remote=True), 12.05848819, .00001)
 
         wrt = ['z', 'x']
         of = ['obj', 'con1', 'con2']
@@ -1884,7 +1886,7 @@ class TestComponentComplexStep(unittest.TestCase):
         with self.assertRaises(RuntimeError) as context:
             model.resetup(setup_mode='reconf')
 
-        msg = "In order to activate complex step during reconfiguration, " \
+        msg = "TestImplCompArrayDense (sub.comp): In order to activate complex step during reconfiguration, " \
               "you need to set 'force_alloc_complex' to True during setup. " \
               "e.g. 'problem.setup(force_alloc_complex=True)'"
         self.assertEqual(str(context.exception), msg)
