@@ -25,10 +25,10 @@ class QuadraticComp(om.ImplicitComponent):
     """
 
     def setup(self):
-        self.add_input('a', val=1.)
+        self.add_input('a', val=1., tags=['tag_a'])
         self.add_input('b', val=1.)
         self.add_input('c', val=1.)
-        self.add_output('x', val=0.)
+        self.add_output('x', val=0., tags=['tag_x'])
 
         self.declare_partials(of='*', wrt='*')
 
@@ -155,20 +155,20 @@ class ImplicitCompTestCase(unittest.TestCase):
         assert_rel_error(self, total_derivs['comp3.x', 'comp1.c'], [[-0.5]])
 
     def test_list_inputs_before_run(self):
-        msg = "Unable to list inputs until model has been run."
+        msg = "Group (<model>): Unable to list inputs until model has been run."
         try:
             self.prob.model.list_inputs()
         except Exception as err:
-            self.assertTrue(msg == str(err))
+            self.assertEqual(str(err), msg)
         else:
             self.fail("Exception expected")
 
     def test_list_outputs_before_run(self):
-        msg = "Unable to list outputs until model has been run."
+        msg = "Group (<model>): Unable to list outputs until model has been run."
         try:
             self.prob.model.list_outputs()
         except Exception as err:
-            self.assertTrue(msg == str(err))
+            self.assertEqual(str(err), msg)
         else:
             self.fail("Exception expected")
 
@@ -190,6 +190,31 @@ class ImplicitCompTestCase(unittest.TestCase):
         self.assertEqual(text.count('comp3.'), 3)
         self.assertEqual(text.count('value'), 1)
 
+    def test_list_inputs_with_tags(self):
+        self.prob.run_model()
+
+        # No tags
+        inputs = self.prob.model.list_inputs(values=False, hierarchical=False, out_stream=None)
+        self.assertEqual(sorted(inputs), [
+            ('comp2.a', {}),
+            ('comp2.b', {}),
+            ('comp2.c', {}),
+            ('comp3.a', {}),
+            ('comp3.b', {}),
+            ('comp3.c', {})
+        ])
+
+        # With tag
+        inputs = self.prob.model.list_inputs(values=False, hierarchical=False, out_stream=None, tags='tag_a')
+        self.assertEqual(sorted(inputs), [
+            ('comp2.a', {}),
+            ('comp3.a', {}),
+        ])
+
+        # Wrong tag
+        inputs = self.prob.model.list_inputs(values=False, hierarchical=False, out_stream=None, tags='tag_wrong')
+        self.assertEqual(sorted(inputs), [])
+
     def test_list_inputs_prom_name(self):
         self.prob.run_model()
 
@@ -224,6 +249,29 @@ class ImplicitCompTestCase(unittest.TestCase):
         self.assertEqual(text.count('varname'), 1)
         self.assertEqual(text.count('value'), 1)
 
+    def test_list_explicit_outputs_with_tags(self):
+        self.prob.run_model()
+
+        # No tags
+        outputs = self.prob.model.list_outputs(explicit=False, hierarchical=False, out_stream=None)
+        self.assertEqual(sorted(outputs), [
+            ('comp2.x', {'value': [3.]}),
+            ('comp3.x', {'value': [3.]}),
+        ])
+
+        # With tag
+        outputs = self.prob.model.list_outputs(explicit=False, hierarchical=False, out_stream=None,
+                                               tags="tag_x")
+        self.assertEqual(sorted(outputs), [
+            ('comp2.x', {'value': [3.]}),
+            ('comp3.x', {'value': [3.]}),
+        ])
+
+        # Wrong tag
+        outputs = self.prob.model.list_outputs(explicit=False, hierarchical=False, out_stream=None,
+                                               tags="tag_wrong")
+        self.assertEqual(sorted(outputs), [])
+
     def test_list_implicit_outputs(self):
         self.prob.run_model()
 
@@ -1049,6 +1097,7 @@ class ListFeatureTestCase(unittest.TestCase):
         prob.model.list_outputs(prom_name=True)
 
     def test_list_return_value(self):
+        # list inputs
         inputs = prob.model.list_inputs(out_stream=None)
         self.assertEqual(sorted(inputs), [
             ('sub.comp2.a', {'value': [1.]}),
@@ -1068,14 +1117,23 @@ class ListFeatureTestCase(unittest.TestCase):
         ])
 
     def test_for_docs_list_no_values(self):
+        # list inputs
         inputs = prob.model.list_inputs(values=False)
-        print(inputs)
 
         # list only explicit outputs
         outputs = prob.model.list_outputs(implicit=False, values=False)
-        print(outputs)
+
+    def test_for_docs_list_includes_excludes(self):
+        # list inputs
+        inputs = prob.model.list_inputs(values=False, includes=['*comp2*',])
+        inputs = prob.model.list_inputs(values=False, excludes=['*comp2*',])
+
+        # list only explicit outputs
+        outputs = prob.model.list_outputs(implicit=False, values=False, includes=['*b',])
+        outputs = prob.model.list_outputs(implicit=False, values=False, excludes=['*b',])
 
     def test_list_no_values(self):
+        # list inputs
         inputs = prob.model.list_inputs(values=False)
         self.assertEqual([n[0] for n in sorted(inputs)], [
             'sub.comp2.a',
@@ -1150,7 +1208,6 @@ class ListFeatureTestCase(unittest.TestCase):
         group.connect('comp1.b', 'sub.comp3.b')
         group.connect('comp1.c', 'sub.comp3.c')
 
-        global prob
         prob = om.Problem(model=group)
         prob.setup()
 
