@@ -201,7 +201,7 @@ class TestScaling(unittest.TestCase):
         model = prob.model
         model.add_subsystem('comp', EComp())
 
-        msg = "'comp': When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'ref'."
+        msg = "EComp (comp): When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'ref'."
         with self.assertRaises(ValueError) as context:
             prob.setup()
         self.assertEqual(_winfix(str(context.exception)), msg)
@@ -214,7 +214,7 @@ class TestScaling(unittest.TestCase):
         model = prob.model
         model.add_subsystem('comp', EComp())
 
-        msg = "'comp': When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'ref0'."
+        msg = "EComp (comp): When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'ref0'."
         with self.assertRaises(ValueError) as context:
             prob.setup()
         self.assertEqual(_winfix(str(context.exception)), msg)
@@ -227,7 +227,7 @@ class TestScaling(unittest.TestCase):
         model = prob.model
         model.add_subsystem('comp', EComp())
 
-        msg = "'comp': When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'res_ref'."
+        msg = "EComp (comp): When adding output 'zz', expected shape (4, 2) but got shape (3, 5) for argument 'res_ref'."
         with self.assertRaises(ValueError) as context:
             prob.setup()
         self.assertEqual(_winfix(str(context.exception)), msg)
