@@ -318,7 +318,7 @@ class TestUnitConversion(unittest.TestCase):
         prob.model.add_subsystem('tgt', om.ExecComp('yy=xx', xx={'value': 0.0, 'units': 'unitless'}))
         prob.model.connect('src.x2', 'tgt.xx')
 
-        msg = "Output 'src.x2' with units of 'degC' is connected to input 'tgt.xx' which has no units."
+        msg = "Group (<model>): Output 'src.x2' with units of 'degC' is connected to input 'tgt.xx' which has no units."
 
         with assert_warning(UserWarning, msg):
             prob.setup()
@@ -612,7 +612,7 @@ class TestUnitConversion(unittest.TestCase):
         with self.assertRaises(Exception) as cm:
             prob.setup()
 
-        expected_msg = "Output units of 'degC' for 'src.x2' are incompatible with input units of 'm' for 'dest.x2'."
+        expected_msg = "Group (<model>): Output units of 'degC' for 'src.x2' are incompatible with input units of 'm' for 'dest.x2'."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -623,7 +623,7 @@ class TestUnitConversion(unittest.TestCase):
         with self.assertRaises(Exception) as cm:
             prob.setup()
 
-        expected_msg = "Output units of 'degC' for 'src.x2' are incompatible with input units of 'm' for 'dest.x2'."
+        expected_msg = "Group (<model>): Output units of 'degC' for 'src.x2' are incompatible with input units of 'm' for 'dest.x2'."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
