@@ -15,6 +15,8 @@ import unittest
 import traceback
 from docutils import nodes
 
+from collections import namedtuple
+
 from six import StringIO
 from six.moves import range, zip, cStringIO as cStringIO
 
@@ -36,6 +38,11 @@ table_name = 'feature_unit_tests'   # name of the table to be queried
 _sub_runner = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'run_sub.py')
 
 
+# an input block consists of a block of code and a tag that marks the end of any
+# output from that code in the output stream (via inserted print('>>>>>#') statements)
+InputBlock = namedtuple('InputBlock', 'code tag')
+
+
 class skipped_or_failed_node(nodes.Element):
     pass
 
@@ -399,18 +406,21 @@ def split_source_into_input_blocks(src):
         List of input code sections.
     """
     input_blocks = []
-
     current_block = []
 
-    for line in src.split('\n'):
+    for line in src.splitlines():
         if 'print(">>>>>' in line:
-            input_blocks.append('\n'.join(current_block))
+            tag = line.split('"')[1]
+            code = '\n'.join(current_block)
+            input_blocks.append(InputBlock(code, tag))
             current_block = []
         else:
             current_block.append(line)
 
-    if current_block and current_block[0]:
-        input_blocks.append('\n'.join(current_block))
+    if len(current_block) > 0:
+        # final input block, with no associated output
+        code = '\n'.join(current_block)
+        input_blocks.append(InputBlock(code, ''))
 
     return input_blocks
 
@@ -430,16 +440,17 @@ def insert_output_start_stop_indicators(src):
         String with output demarked.
     """
     lines = src.split('\n')
-    print_producing = ['.setup(',
-                       'print(',
-                       '.run_model(',
-                       '.run_driver(',
-                       '.check_partials(',
-                       '.check_totals(',
-                       '.list_inputs(',
-                       '.list_outputs(',
-                       '.list_problem_vars(',
-                       ]
+    print_producing = [
+        'print(',
+        '.setup(',
+        '.run_model(',
+        '.run_driver(',
+        '.check_partials(',
+        '.check_totals(',
+        '.list_inputs(',
+        '.list_outputs(',
+        '.list_problem_vars(',
+    ]
 
     newlines = []
     input_block_number = 0
@@ -489,32 +500,41 @@ def insert_output_start_stop_indicators(src):
     return '\n'.join(newlines)
 
 
-def clean_up_empty_output_blocks(input_blocks, output_blocks):
-    """Some of the blocks do not generate output. We only want to have
-    input blocks that have outputs.
+def consolidate_input_blocks(input_blocks, output_blocks):
     """
+    Merge any input blocks for which there is no corresponding output
+    with subsequent blocks that do have output.
 
+    Remove any leading and trailing blank lines from all input blocks.
+    """
     new_input_blocks = []
-    new_output_blocks = []
-    current_in_block = ''
+    new_block = ''
 
-    for in_block, out_block in zip(input_blocks, output_blocks):
-        if current_in_block and not current_in_block.endswith('\n'):
-            current_in_block += '\n'
-        current_in_block += in_block
-        if out_block:
-            current_in_block = remove_leading_trailing_whitespace_lines(current_in_block)
-            out_block = remove_leading_trailing_whitespace_lines(out_block)
-            new_input_blocks.append(current_in_block)
-            new_output_blocks.append(out_block)
-            current_in_block = ''
+    for (code, tag) in input_blocks:
+        if tag not in output_blocks:
+            # no output, add to new consolidated block
+            if new_block and not new_block.endswith('\n'):
+                new_block += '\n'
+            new_block += code
+        elif new_block:
+            # add current input to new consolidated block and save
+            if new_block and not new_block.endswith('\n'):
+                new_block += '\n'
+            new_block += code
+            new_block = remove_leading_trailing_whitespace_lines(new_block)
+            new_input_blocks.append(InputBlock(new_block, tag))
+            new_block = ''
+        else:
+            # just strip leading/trailing from input block
+            code = remove_leading_trailing_whitespace_lines(code)
+            new_input_blocks.append(InputBlock(code, tag))
 
-    # if there was no output, return the one input block and empty output block
-    if current_in_block:
-        new_input_blocks.append(current_in_block)
-        new_output_blocks.append('')
+    # trailing input with no corresponding output
+    if new_block:
+        new_block = remove_leading_trailing_whitespace_lines(new_block)
+        new_input_blocks.append(InputBlock(new_block, ''))
 
-    return new_input_blocks, new_output_blocks
+    return new_input_blocks
 
 
 def extract_output_blocks(run_output):
@@ -528,31 +548,30 @@ def extract_output_blocks(run_output):
 
     Returns
     -------
-    list of str
-        List containing output text blocks.
+    dict
+        output blocks keyed on tags like ">>>>>4"
     """
-
-    # Look for start and end lines that look like this:
-    #  <<<<<4
-    #  >>>>>4
-
     if isinstance(run_output, list):
         return sync_multi_output_blocks(run_output)
 
-    output_blocks = []
+    output_blocks = {}
     output_block = None
 
     for line in run_output.splitlines():
         if output_block is None:
             output_block = []
         if line[:5] == '>>>>>':
-            output_blocks.append('\n'.join(output_block))
+            output = ('\n'.join(output_block)).strip()
+            if output:
+                output_blocks[line] = output
             output_block = None
         else:
             output_block.append(line)
 
     if output_block is not None:
-        output_blocks.append('\n'.join(output_block))
+        # It is possible to have trailing output
+        # (e.g. if the last print_producing statement is in a try block)
+        output_blocks['Trailing'] = output_block
 
     return output_blocks
 
@@ -566,7 +585,7 @@ def strip_header(src):
     Parameters
     ----------
     src : str
-        sourec code for method
+        source code
     """
     lines = src.split('\n')
     first_len = None
@@ -591,7 +610,7 @@ def dedent(src):
     Parameters
     ----------
     src : str
-        sourec code for method
+        source code
     """
 
     lines = src.split('\n')
@@ -604,30 +623,37 @@ def dedent(src):
     return ''
 
 
-def sync_multi_output_blocks(blocks):
+def sync_multi_output_blocks(run_output):
     """
     Combine output from different procs into the same output blocks.
 
     Parameters
     ----------
-    blocks : list of str
+    run_output : list of dict
         List of outputs from individual procs.
 
     Returns
     -------
-    list of list of str
-        List of synced output blocks from all procs.
+    dict
+        Synced output blocks from all procs.
     """
-    if blocks:
-        split_blocks = [extract_output_blocks(b) for b in blocks]
-        n_out_blocks = len(split_blocks[0])
-        synced_blocks = []
-        for i in range(n_out_blocks):
-            synced_blocks.append('\n'.join(["(rank %d) %s" % (j, m[i])
-                for j, m in enumerate(split_blocks) if m[i]]))
+    if run_output:
+        # for each proc's run output, get a dict of output blocks keyed by tag
+        proc_output_blocks = [extract_output_blocks(outp) for outp in run_output]
+
+        synced_blocks = {}
+
+        for i, outp in enumerate(proc_output_blocks):
+            for tag in outp:
+                if outp[tag].strip():
+                    if tag in synced_blocks:
+                        synced_blocks[tag] += "(rank %d) %s\n" % (i, outp[tag])
+                    else:
+                        synced_blocks[tag] = "(rank %d) %s\n" % (i, outp[tag])
+
         return synced_blocks
     else:
-        return []
+        return {}
 
 
 def run_code(code_to_run, path, module=None, cls=None, shows_plot=False, imports_not_required=False):
@@ -740,10 +766,11 @@ def run_code(code_to_run, path, module=None, cls=None, shows_plot=False, imports
 
                 try:
                     exec(code_to_run, globals_dict)
-                except Exception:
-                    # print code (with line numbers) to facilitate debugging
-                    for n, line in enumerate(code_to_run.split('\n')):
-                        print('%4d: %s' % (n, line), file=stderr)
+                except Exception as err:
+                    # for actual errors, print code (with line numbers) to facilitate debugging
+                    if not isinstance(err, unittest.SkipTest):
+                        for n, line in enumerate(code_to_run.split('\n')):
+                            print('%4d: %s' % (n, line), file=stderr)
                     raise
                 finally:
                     sys.stdout = stdout
@@ -779,17 +806,33 @@ def get_skip_output_node(output):
 
 
 def get_interleaved_io_nodes(input_blocks, output_blocks):
+    """
+    Parameters
+    ----------
+    input_blocks : list of tuple
+        Each tuple is a block of code and the tag marking it's output.
+
+    output_blocks : dict
+        Output blocks keyed on tag.
+    """
     nodelist = []
     n = 1
-    output_blocks = [cgiesc.escape(ob) for ob in output_blocks]
-    for input_block, output_block in zip(input_blocks, output_blocks):
-        input_node = nodes.literal_block(input_block, input_block)
+
+    for (code, tag) in input_blocks:
+        input_node = nodes.literal_block(code, code)
         input_node['language'] = 'python'
         nodelist.append(input_node)
-        if len(output_block) > 0:
-            output_node = in_or_out_node(kind="Out", number=n, text=output_block)
-            nodelist.append(output_node)
+        if tag and tag in output_blocks:
+            outp = cgiesc.escape(output_blocks[tag])
+            if (outp.strip()):
+                output_node = in_or_out_node(kind="Out", number=n, text=outp)
+                nodelist.append(output_node)
         n += 1
+
+    if 'Trailing' in output_blocks:
+        output_node = in_or_out_node(kind="Out", number=n, text=output_blocks['Trailing'])
+        nodelist.append(output_node)
+
     return nodelist
 
 
