@@ -1,9 +1,10 @@
 """ Test the Jacobian objects."""
 
 import itertools
+import sys
 import unittest
 
-from six import assertRaisesRegex
+from six import assertRaisesRegex, StringIO
 from six.moves import range
 
 import numpy as np
@@ -15,8 +16,7 @@ from openmdao.api import IndepVarComp, Group, Problem, \
                          LinearBlockGS, DirectSolver
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.test_suite.components.paraboloid import Paraboloid
-from openmdao.test_suite.components.sellar import SellarDis1withDerivatives, \
-     SellarDis2withDerivatives
+from openmdao.api import ScipyOptimizeDriver
 
 try:
     from parameterized import parameterized
@@ -149,6 +149,41 @@ class ExplicitSetItemComp(ExplicitComponent):
         partials['out', 'in'] = self._constructor(self._value)
 
 
+class SimpleCompWithPrintPartials(ExplicitComponent):
+
+    def setup(self):
+        self.add_input('x', val=0.0)
+        self.add_input('y', val=0.0)
+
+        self.add_output('f_xy', val=0.0, upper=1.0)
+
+        self.declare_partials(of='*', wrt='*')
+
+        self.count = 0
+        self.partials_name_pairs = []
+        self.partials_values = []
+
+    def compute(self, inputs, outputs):
+        x = inputs['x']
+        y = inputs['y']
+        outputs['f_xy'] = (x-3.0)**2 + x*y + (y+4.0)**2 - 3.0
+
+    def compute_partials(self, inputs, partials):
+        x = inputs['x']
+        y = inputs['y']
+
+        partials['f_xy', 'x'] = 2.0*x - 6.0 + y
+        partials['f_xy', 'y'] = 2.0*y + 8.0 + x
+
+        if self.count < 1:  # Only want to save these this once for the test
+            for k in partials:
+                self.partials_name_pairs.append(k)
+
+            for k, v in partials.items():
+                self.partials_values.append((k,v))
+
+        self.count += 1
+
 def arr2list(arr):
     """Convert a numpy array to a 'sparse' list."""
     data = []
@@ -566,7 +601,7 @@ class TestJacobian(unittest.TestCase):
         model = prob.model
         model.add_subsystem('comp', Comp1())
 
-        msg = "comp: declare_partials has been called with rows and cols, which" + \
+        msg = "Comp1 \(comp\): d\(y\)/d\(x\): declare_partials has been called with rows and cols, which" + \
               " should be arrays of equal length, but rows is length 2 while " + \
               "cols is length 1."
         with assertRaisesRegex(self, RuntimeError, msg):
@@ -576,7 +611,7 @@ class TestJacobian(unittest.TestCase):
         model = prob.model
         model.add_subsystem('comp', Comp2())
 
-        msg = "comp: declare_partials has been called with rows and cols, which" + \
+        msg = "Comp2 \(comp\): d\(y\)/d\(x\): declare_partials has been called with rows and cols, which" + \
             " should be arrays of equal length, but rows is length 1 while " + \
             "cols is length 2."
         with assertRaisesRegex(self, RuntimeError, msg):
@@ -784,6 +819,40 @@ class TestJacobian(unittest.TestCase):
         J = prob.compute_totals(of=['G1.C1.z'], wrt=['indeps.x'])
         assert_rel_error(self, J['G1.C1.z', 'indeps.x'], np.eye(10)*5.0, .0001)
 
+    def test_dict_properties(self):
+        # Make sure you can use the partials variable passed to compute_partials as a dict
+        prob = Problem()
+
+        indeps = prob.model.add_subsystem('indeps', IndepVarComp(), promotes=['*'])
+        indeps.add_output('x', .5)
+        indeps.add_output('y', 10.0)
+        comp = SimpleCompWithPrintPartials()
+        prob.model.add_subsystem('paraboloid', comp, promotes_inputs=['x', 'y'])
+
+        prob.driver = ScipyOptimizeDriver()
+        prob.driver.options['optimizer'] = 'SLSQP'
+
+        prob.model.add_design_var('x', lower=-50, upper=50)
+        prob.model.add_design_var('y', lower=-50, upper=50)
+        prob.model.add_objective('paraboloid.f_xy')
+
+        prob.setup()
+        prob.run_driver()
+
+        expected = [
+            (('paraboloid.f_xy', 'paraboloid.f_xy'),[-1.]),
+            (('paraboloid.f_xy', 'paraboloid.x'),[[0.]]),
+            (('paraboloid.f_xy', 'paraboloid.y'),[[0.]]),
+        ]
+        self.assertEqual(sorted(comp.partials_name_pairs), sorted(e[0] for e in sorted(expected)))
+
+        self.assertEqual(sorted(comp.partials_name_pairs),
+                         sorted(e[0] for e in sorted(expected)))
+        for act, exp in zip(
+                [e[1] for e in sorted(comp.partials_values)],
+                [e[1] for e in sorted(expected)],
+                ):
+            assert_rel_error(self,act,exp, 1e-5)
 
 class MySparseComp(ExplicitComponent):
     def setup(self):
@@ -902,5 +971,71 @@ class OverlappingPartialsTestCase(unittest.TestCase):
                                                  [ 9.,  8., -1.,  0.],
                                                  [ 5.,  10.,  0., -1.]]))
 
+
+class MaskingTestCase(unittest.TestCase):
+    def test_csc_masking(self):
+        class CCBladeResidualComp(ImplicitComponent):
+
+            def initialize(self):
+                self.options.declare('num_nodes', types=int)
+                self.options.declare('num_radial', types=int)
+
+            def setup(self):
+                num_nodes = self.options['num_nodes']
+                num_radial = self.options['num_radial']
+
+                self.add_input('chord', shape=(1, num_radial))
+                self.add_input('theta', shape=(1, num_radial))
+
+                self.add_output('phi', lower=-0.5*np.pi, upper=0.0,
+                                shape=(num_nodes, num_radial))
+                self.add_output('Tp', shape=(num_nodes, num_radial))
+
+                of_names = ('phi', 'Tp')
+                row_col = np.arange(num_radial)
+
+                for name in of_names:
+                    self.declare_partials(name, 'chord', rows=row_col, cols=row_col)
+                    self.declare_partials(name, 'theta', rows=row_col, cols=row_col, val=0.0)
+                    self.declare_partials(name, 'phi', rows=row_col, cols=row_col)
+
+                self.declare_partials('Tp', 'Tp', rows=row_col, cols=row_col, val=1.)
+
+            def linearize(self, inputs, outputs, partials):
+
+                partials['phi', 'chord'] = np.array([1., 2, 3, 4])
+                partials['phi', 'phi'] = np.array([5., 6, 7, 8])
+
+                partials['Tp', 'chord'] = np.array([9., 10, 11, 12])
+                partials['Tp', 'phi'] = np.array([13., 14, 15, 16])
+
+
+        prob = Problem()
+        model = prob.model
+
+        comp = IndepVarComp()
+        comp.add_output('chord', val=np.ones((4, )))
+        model.add_subsystem('indep_var_comp', comp, promotes=['*'])
+
+        comp = CCBladeResidualComp(num_nodes=1, num_radial=4, assembled_jac_type='csc')
+
+        comp.linear_solver = DirectSolver(assemble_jac=True)
+        model.add_subsystem('ccblade_comp', comp, promotes_inputs=['chord'], promotes_outputs=['Tp'])
+
+
+        prob.setup(mode='fwd')
+        prob.run_model()
+        totals = prob.compute_totals(of=['Tp'], wrt=['chord'], return_format='array')
+
+        expected = np.array([
+        [-6.4,0.,0.,0.],
+        [ 0.,-5.33333333,0.,0.],
+        [ 0.,0.,-4.57142857,0.],
+        [ 0.,0.,0.,-4.]]
+        )
+
+        np.testing.assert_allclose(totals, expected)
+
+
 if __name__ == '__main__':
     unittest.main()
