@@ -10,12 +10,15 @@ from collections import OrderedDict
 
 import numpy as np
 
-from openmdao.utils.record_util import json_to_np_array
 from openmdao.recorders.sqlite_recorder import blob_to_array
-
+from openmdao.utils.record_util import deserialize, get_source_system
 from openmdao.utils.variable_table import write_var_table
+from openmdao.utils.general_utils import warn_deprecation, make_set, \
+    var_name_match_includes_excludes
+from openmdao.utils.units import get_conversion
 
 _DEFAULT_OUT_STREAM = object()
+_AMBIGOUS_PROM_NAME = object()
 
 
 class Case(object):
@@ -26,8 +29,8 @@ class Case(object):
     ----------
     source : str
         The unique id of the system/solver/driver/problem that did the recording.
-    iteration_coordinate : str
-        The full unique identifier for this iteration.
+    name : str
+        The unique identifier for this case.
     parent : str
         The iteration coordinate of the parent case for this iteration if any, else None.
     counter : int
@@ -38,13 +41,13 @@ class Case(object):
         Success flag for the case.
     msg : str
         Message associated with the case.
-    outputs : PromotedToAbsoluteMap
+    outputs : PromAbsDict
         Map of outputs to values recorded.
-    inputs : PromotedToAbsoluteMap or None
+    inputs : PromAbsDict or None
         Map of inputs to values recorded, None if not recorded.
-    residuals : PromotedToAbsoluteMap or None
+    residuals : PromAbsDict or None
         Map of outputs to residuals recorded, None if not recorded.
-    jacobian : PromotedToAbsoluteMap or None
+    jacobian : PromAbsDict or None
         Map of (output, input) to derivatives recorded, None if not recorded.
     parent : str
         The full unique identifier for the parent this iteration.
@@ -58,13 +61,13 @@ class Case(object):
         Dictionary mapping absolute names of all variables to promoted names.
     _abs2meta : dict
         Dictionary mapping absolute names of all variables to variable metadata.
-    _voi_meta : dict
-        Dictionary mapping absolute names of variables of interest to variable metadata.
+    _var_info : dict
+        Dictionary with information about variables (scaling, indices, execution order).
     _format_version : int
         A version number specifying the format of array data, if not numpy arrays.
     """
 
-    def __init__(self, source, data, prom2abs, abs2prom, abs2meta, voi_meta, data_format=None):
+    def __init__(self, source, data, prom2abs, abs2prom, abs2meta, var_info, data_format=None):
         """
         Initialize.
 
@@ -80,8 +83,8 @@ class Case(object):
             Dictionary mapping absolute names of all variables to promoted names.
         abs2meta : dict
             Dictionary mapping absolute names of all variables to variable metadata.
-        voi_meta : dict
-            Dictionary mapping absolute names of variables of interest to variable metadata.
+        var_info : dict
+            Dictionary with information about variables (scaling, indices, execution order).
         data_format : int
             A version number specifying the format of array data, if not numpy arrays.
         """
@@ -89,17 +92,17 @@ class Case(object):
         self._format_version = data_format
 
         if 'iteration_coordinate' in data.keys():
-            self.iteration_coordinate = data['iteration_coordinate']
-            parts = self.iteration_coordinate.split('|')
+            self.name = data['iteration_coordinate']
+            parts = self.name.split('|')
             if len(parts) > 2:
                 self.parent = '|'.join(parts[:-2])
             else:
                 self.parent = None
         elif 'case_name' in data.keys():
-            self.iteration_coordinate = data['case_name']  # problem cases
+            self.name = data['case_name']  # problem cases
             self.parent = None
         else:
-            self.iteration_coordinate = None
+            self.name = None
             self.parent = None
 
         self.counter = data['counter']
@@ -127,7 +130,7 @@ class Case(object):
 
         if 'inputs' in data.keys():
             if data_format >= 3:
-                inputs = json_to_np_array(data['inputs'], abs2meta)
+                inputs = deserialize(data['inputs'], abs2meta)
             elif data_format in (1, 2):
                 inputs = blob_to_array(data['inputs'])
                 if type(inputs) is np.ndarray and not inputs.shape:
@@ -135,11 +138,11 @@ class Case(object):
             else:
                 inputs = data['inputs']
             if inputs is not None:
-                self.inputs = PromotedToAbsoluteMap(inputs[0], prom2abs, abs2prom, output=False)
+                self.inputs = PromAbsDict(inputs, prom2abs, abs2prom, output=False)
 
         if 'outputs' in data.keys():
             if data_format >= 3:
-                outputs = json_to_np_array(data['outputs'], abs2meta)
+                outputs = deserialize(data['outputs'], abs2meta)
             elif self._format_version in (1, 2):
                 outputs = blob_to_array(data['outputs'])
                 if type(outputs) is np.ndarray and not outputs.shape:
@@ -147,11 +150,11 @@ class Case(object):
             else:
                 outputs = data['outputs']
             if outputs is not None:
-                self.outputs = PromotedToAbsoluteMap(outputs[0], prom2abs, abs2prom)
+                self.outputs = PromAbsDict(outputs, prom2abs, abs2prom)
 
         if 'residuals' in data.keys():
             if data_format >= 3:
-                residuals = json_to_np_array(data['residuals'], abs2meta)
+                residuals = deserialize(data['residuals'], abs2meta)
             elif data_format in (1, 2):
                 residuals = blob_to_array(data['residuals'])
                 if type(residuals) is np.ndarray and not residuals.shape:
@@ -159,7 +162,7 @@ class Case(object):
             else:
                 residuals = data['residuals']
             if residuals is not None:
-                self.residuals = PromotedToAbsoluteMap(residuals[0], prom2abs, abs2prom)
+                self.residuals = PromAbsDict(residuals, prom2abs, abs2prom)
 
         if 'jacobian' in data.keys():
             if data_format >= 2:
@@ -169,7 +172,7 @@ class Case(object):
             else:
                 jacobian = data['jacobian']
             if jacobian is not None:
-                self.jacobian = PromotedToAbsoluteMap(jacobian[0], prom2abs, abs2prom, output=True)
+                self.jacobian = PromAbsDict(jacobian, prom2abs, abs2prom, output=True)
 
         # save var name & meta dict references for use by self._get_variables_of_type()
         self._prom2abs = prom2abs
@@ -177,7 +180,20 @@ class Case(object):
         self._abs2meta = abs2meta
 
         # save VOI dict reference for use by self._scale()
-        self._voi_meta = voi_meta
+        self._var_info = var_info
+
+    @property
+    def iteration_coordinate(self):
+        """
+        Deprecate the 'iteration_coordinate' attribute.
+
+        Returns
+        -------
+        str
+            The unique identifier for this case.
+        """
+        warn_deprecation("'iteration_coordinate' has been deprecated. Use 'name' instead.")
+        return self.name
 
     def __str__(self):
         """
@@ -188,7 +204,111 @@ class Case(object):
         str
             String representation of the case.
         """
-        return ' '.join([self.source, self.iteration_coordinate, str(self.outputs)])
+        return ' '.join([self.source, self.name, str(self.outputs)])
+
+    def __getitem__(self, name):
+        """
+        Get an output/input variable.
+
+        Parameters
+        ----------
+        name : str
+            Promoted or relative variable name in the root system's namespace.
+
+        Returns
+        -------
+        float or ndarray or any python object
+            the requested output/input variable.
+        """
+        if self.outputs is not None:
+            try:
+                return self.outputs[name]
+            except KeyError:
+                if self.inputs is not None:
+                    return self.inputs[name]
+        elif self.inputs is not None:
+            return self.inputs[name]
+
+        raise KeyError('Variable name "%s" not found.' % name)
+
+    def get_val(self, name, units=None, indices=None):
+        """
+        Get an output/input variable.
+
+        Function is used if you want to specify display units.
+
+        Parameters
+        ----------
+        name : str
+            Promoted or relative variable name in the root system's namespace.
+        units : str, optional
+            Units to convert to before upon return.
+        indices : int or list of ints or tuple of ints or int ndarray or Iterable or None, optional
+            Indices or slice to return.
+
+        Returns
+        -------
+        float or ndarray
+            The requested output/input variable.
+        """
+        val = self[name]
+
+        if indices is not None:
+            val = val[indices]
+
+        if units is not None:
+            base_units = self._get_units(name)
+
+            if base_units is None:
+                msg = "Can't express variable '{}' with units of 'None' in units of '{}'."
+                raise TypeError(msg.format(name, units))
+
+            try:
+                scale, offset = get_conversion(base_units, units)
+            except TypeError:
+                msg = "Can't express variable '{}' with units of '{}' in units of '{}'."
+                raise TypeError(msg.format(name, base_units, units))
+
+            val = (val + offset) * scale
+
+        return val
+
+    def _get_units(self, name):
+        """
+        Get the units for a variable name.
+
+        Parameters
+        ----------
+        name : str
+            Promoted or relative variable name in the root system's namespace.
+
+        Returns
+        -------
+        str
+            Unit string.
+        """
+        meta = self._abs2meta
+
+        if name in meta:
+            return meta[name]['units']
+
+        proms = self._prom2abs
+
+        if name in proms['output']:
+            abs_name = proms['output'][name][0]
+            return meta[abs_name]['units']
+
+        elif name in proms['input']:
+            if len(proms['input'][name]) > 1:
+                # The promoted name maps to multiple absolute names, require absolute name.
+                msg = "Can't get units for the promoted name '%s' because it refers to " + \
+                      "multiple inputs: %s. Access the units using an absolute path name."
+                raise RuntimeError(msg % (name, str(proms['input'][name])))
+
+            abs_name = proms['input'][name][0]
+            return meta[abs_name]['units']
+
+        raise KeyError('Variable name "{}" not found.'.format(name))
 
     def get_design_vars(self, scaled=True, use_indices=True):
         """
@@ -197,18 +317,18 @@ class Case(object):
         Parameters
         ----------
         scaled : bool
-            The unique id of the system/solver/driver/problem that did the recording.
+            If True, then return scaled values.
         use_indices : bool
-            The full unique identifier for this iteration.
+            If True, apply indices.
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their values.
         """
         vals = self._get_variables_of_type('desvar')
         if scaled:
-            return self._apply_voi_meta(vals, scaled, use_indices)
+            return self._apply_var_settings(vals, scaled, use_indices)
         else:
             return vals
 
@@ -219,18 +339,18 @@ class Case(object):
         Parameters
         ----------
         scaled : bool
-            The unique id of the system/solver/driver/problem that did the recording.
+            If True, then return scaled values.
         use_indices : bool
-            The full unique identifier for this iteration.
+            If True, apply indices.
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their values.
         """
         vals = self._get_variables_of_type('objective')
         if scaled:
-            return self._apply_voi_meta(vals, scaled, use_indices)
+            return self._apply_var_settings(vals, scaled, use_indices)
         else:
             return vals
 
@@ -241,18 +361,18 @@ class Case(object):
         Parameters
         ----------
         scaled : bool
-            The unique id of the system/solver/driver/problem that did the recording.
+            If True, then return scaled values.
         use_indices : bool
-            The full unique identifier for this iteration.
+            If True, apply indices.
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their values.
         """
         vals = self._get_variables_of_type('constraint')
         if scaled:
-            return self._apply_voi_meta(vals, scaled, use_indices)
+            return self._apply_var_settings(vals, scaled, use_indices)
         else:
             return vals
 
@@ -263,26 +383,31 @@ class Case(object):
         Parameters
         ----------
         scaled : bool
-            The unique id of the system/solver/driver/problem that did the recording.
+            If True, then return scaled values.
         use_indices : bool
-            The full unique identifier for this iteration.
+            If True, apply indices.
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their values.
         """
         vals = self._get_variables_of_type('response')
         if scaled:
-            return self._apply_voi_meta(vals, scaled, use_indices)
+            return self._apply_var_settings(vals, scaled, use_indices)
         else:
             return vals
 
     def list_inputs(self,
                     values=True,
+                    prom_name=False,
                     units=False,
+                    shape=False,
                     hierarchical=True,
                     print_arrays=False,
+                    tags=None,
+                    includes=None,
+                    excludes=None,
                     out_stream=_DEFAULT_OUT_STREAM):
         """
         Return and optionally log a list of input names and other optional information.
@@ -293,8 +418,13 @@ class Case(object):
         ----------
         values : bool, optional
             When True, display/return input values. Default is True.
+        prom_name : bool, optional
+            When True, display/return the promoted name of the variable.
+            Default is False.
         units : bool, optional
             When True, display/return units. Default is False.
+        shape : bool, optional
+            When True, display/return the shape of the value. Default is False.
         hierarchical : bool, optional
             When True, human readable output shows variables in hierarchical format.
         print_arrays : bool, optional
@@ -303,6 +433,16 @@ class Case(object):
             When True, also display full values of the ndarray below the row. Format is affected
             by the values set with numpy.set_printoptions
             Default is False.
+        tags : str or list of strs
+            User defined tags that can be used to filter what gets listed. Only inputs with the
+            given tags will be listed.
+            Default is None, which means there will be no filtering based on tags.
+        includes : None or list_like
+            List of glob patterns for pathnames to include in the check. Default is None, which
+            includes all components in the model.
+        excludes : None or list_like
+            List of glob patterns for pathnames to exclude from the check. Default is None, which
+            excludes nothing.
         out_stream : file-like object
             Where to send human readable output. Default is sys.stdout.
             Set to None to suppress.
@@ -313,28 +453,35 @@ class Case(object):
             list of input names and other optional information about those inputs
         """
         meta = self._abs2meta
-        inp_vars = {}
         inputs = []
 
         if self.inputs is not None:
-            for abs_name in self.inputs.absolute_names():
-                if abs_name not in inp_vars:
-                    inp_vars[abs_name] = {'value': self.inputs[abs_name]}
+            for var_name in self.inputs.absolute_names():
+                # Filter based on tags
+                if tags and not (make_set(tags) & make_set(meta[var_name]['tags'])):
+                    continue
 
-        if inp_vars is not None and len(inp_vars) > 0:
-            for name in inp_vars:
-                outs = {}
+                if not var_name_match_includes_excludes(var_name,
+                                                        self._abs2prom['input'][var_name],
+                                                        includes, excludes):
+                    continue
+
+                var_meta = {}
                 if values:
-                    outs['value'] = inp_vars[name]['value']
+                    var_meta['value'] = self.inputs[var_name]
+                if prom_name:
+                    var_meta['prom_name'] = self._abs2prom['input'][var_name]
                 if units:
-                    outs['units'] = meta[name]['units']
-                inputs.append((name, outs))
+                    var_meta['units'] = meta[var_name]['units']
+                if shape:
+                    var_meta['shape'] = self.inputs[var_name].shape
+                inputs.append((var_name, var_meta))
 
         if out_stream == _DEFAULT_OUT_STREAM:
             out_stream = sys.stdout
 
         if out_stream:
-            if len(inp_vars) is 0:
+            if self.inputs is None or len(self.inputs) is 0:
                 out_stream.write('WARNING: Inputs not recorded. Make sure your recording ' +
                                  'settings have record_inputs set to True\n')
 
@@ -354,6 +501,9 @@ class Case(object):
                      scaling=False,
                      hierarchical=True,
                      print_arrays=False,
+                     tags=None,
+                     includes=None,
+                     excludes=None,
                      out_stream=_DEFAULT_OUT_STREAM):
         """
         Return and optionally log a list of output names and other optional information.
@@ -393,6 +543,16 @@ class Case(object):
             When True, also display full values of the ndarray below the row. Format  is affected
             by the values set with numpy.set_printoptions
             Default is False.
+        tags : str or list of strs
+            User defined tags that can be used to filter what gets listed. Only inputs with the
+            given tags will be listed.
+            Default is None, which means there will be no filtering based on tags.
+        includes : None or list_like
+            List of glob patterns for pathnames to include in the check. Default is None, which
+            includes all components in the model.
+        excludes : None or list_like
+            List of glob patterns for pathnames to exclude from the check. Default is None, which
+            excludes nothing.
         out_stream : file-like
             Where to send human readable output. Default is sys.stdout.
             Set to None to suppress.
@@ -405,49 +565,52 @@ class Case(object):
         meta = self._abs2meta
         expl_outputs = []
         impl_outputs = []
-        out_vars = {}
 
-        for abs_name in self.outputs.absolute_names():
-            out_vars[abs_name] = {'value': self.outputs[abs_name]}
-            if self.residuals and abs_name in self.residuals.absolute_names():
-                out_vars[abs_name]['residuals'] = self.residuals[abs_name]
-            else:
-                out_vars[abs_name]['residuals'] = 'Not Recorded'
+        for var_name in self.outputs.absolute_names():
+            # Filter based on tags
+            if tags and not (make_set(tags) & make_set(meta[var_name]['tags'])):
+                continue
 
-        if len(out_vars) > 0:
-            for name in out_vars:
-                if residuals_tol and \
-                   out_vars[name]['residuals'] is not 'Not Recorded' and \
-                   np.linalg.norm(out_vars[name]['residuals']) < residuals_tol:
+            if not var_name_match_includes_excludes(var_name, self._abs2prom['output'][var_name],
+                                                    includes, excludes):
+                continue
+
+            # check if residuals were recorded, skip if within specifed tolerance
+            if self.residuals and var_name in self.residuals.absolute_names():
+                resids = self.residuals[var_name]
+                if residuals_tol and np.linalg.norm(resids) < residuals_tol:
                     continue
-                outs = {}
-                if values:
-                    outs['value'] = out_vars[name]['value']
-                if prom_name:
-                    outs['prom_name'] = self._var_abs2prom['output'][name]
-                if residuals:
-                    outs['resids'] = out_vars[name]['residuals']
-                if units:
-                    outs['units'] = meta[name]['units']
-                if shape:
-                    outs['shape'] = out_vars[name]['value'].shape
-                if bounds:
-                    outs['lower'] = meta[name]['lower']
-                    outs['upper'] = meta[name]['upper']
-                if scaling:
-                    outs['ref'] = meta[name]['ref']
-                    outs['ref0'] = meta[name]['ref0']
-                    outs['res_ref'] = meta[name]['res_ref']
-                if meta[name]['explicit']:
-                    expl_outputs.append((name, outs))
-                else:
-                    impl_outputs.append((name, outs))
+            else:
+                resids = 'Not Recorded'
+
+            var_meta = {}
+            if values:
+                var_meta['value'] = self.outputs[var_name]
+            if prom_name:
+                var_meta['prom_name'] = self._abs2prom['output'][var_name]
+            if residuals:
+                var_meta['resids'] = resids
+            if units:
+                var_meta['units'] = meta[var_name]['units']
+            if shape:
+                var_meta['shape'] = self.outputs[var_name].shape
+            if bounds:
+                var_meta['lower'] = meta[var_name]['lower']
+                var_meta['upper'] = meta[var_name]['upper']
+            if scaling:
+                var_meta['ref'] = meta[var_name]['ref']
+                var_meta['ref0'] = meta[var_name]['ref0']
+                var_meta['res_ref'] = meta[var_name]['res_ref']
+            if meta[var_name]['explicit']:
+                expl_outputs.append((var_name, var_meta))
+            else:
+                impl_outputs.append((var_name, var_meta))
 
         if out_stream == _DEFAULT_OUT_STREAM:
             out_stream = sys.stdout
 
         if out_stream:
-            if len(out_vars) is 0:
+            if self.outputs is None or len(self.outputs) is 0:
                 out_stream.write('WARNING: Outputs not recorded. Make sure your recording ' +
                                  'settings have record_outputs set to True\n')
 
@@ -499,7 +662,25 @@ class Case(object):
         for name, vals in var_data:
             var_dict[name] = vals
 
-        write_var_table('model', var_dict.keys(), var_type, var_dict,
+        # determine pathname of the system
+        if self.source in ('root', 'driver', 'problem', 'root.nonlinear_solver'):
+            pathname = ''
+        elif '|' in self.source:
+            pathname = get_source_system(self.source)
+        else:
+            pathname = self.source.replace('root.', '')
+            if pathname.endswith('.nonlinear_solver'):
+                pathname = pathname[:-17]  # len('.nonlinear_solver') == 17
+
+        # vars should be in execution order
+        if 'execution_order' in self._var_info:
+            var_order = self._var_info['execution_order']
+            var_list = [var_name for var_name in var_order if var_name in var_dict]
+        else:
+            # don't have execution order, just sort for determinism
+            var_list = sorted(var_dict.keys())
+
+        write_var_table(pathname, var_list, var_type, var_dict,
                         hierarchical, print_arrays, out_stream)
 
     def _get_variables_of_type(self, var_type):
@@ -514,40 +695,40 @@ class Case(object):
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their values.
         """
         if self.outputs is None:
-            return PromotedToAbsoluteMap({}, self._prom2abs, self._abs2prom)
+            return PromAbsDict({}, self._prom2abs, self._abs2prom)
 
         ret_vars = {}
         for var in self.outputs.absolute_names():
             if var_type in self._abs2meta[var]['type']:
                 ret_vars[var] = self.outputs[var]
 
-        return PromotedToAbsoluteMap(ret_vars, self._prom2abs, self._abs2prom)
+        return PromAbsDict(ret_vars, self._prom2abs, self._abs2prom)
 
-    def _apply_voi_meta(self, vals, scaled=True, use_indices=True):
+    def _apply_var_settings(self, vals, scaled=True, use_indices=True):
         """
-        Scale the values array and apply indices from _voi_meta per the arguments.
+        Scale the values array and apply indices from _var_info per the arguments.
 
         Parameters
         ----------
-        vals : PromotedToAbsoluteMap
+        vals : PromAbsDict
             Map of variables to their values.
         scaled : bool
-            The unique id of the system/solver/driver/problem that did the recording.
+            If True, then return scaled values.
         use_indices : bool
-            The full unique identifier for this iteration.
+            If True, apply indices.
 
         Returns
         -------
-        PromotedToAbsoluteMap
+        PromAbsDict
             Map of variables to their scaled values.
         """
         for name in vals.absolute_names():
-            if name in self._voi_meta:
-                meta = self._voi_meta[name]
+            if name in self._var_info:
+                meta = self._var_info[name]
                 if scaled:
                     # physical to scaled
                     if meta['adder'] is not None:
@@ -561,7 +742,7 @@ class Case(object):
         return vals
 
 
-class PromotedToAbsoluteMap(dict):
+class PromAbsDict(dict):
     """
     A dictionary that enables accessing values via absolute or promoted variable names.
 
@@ -594,7 +775,7 @@ class PromotedToAbsoluteMap(dict):
         output : bool
             True if this should map using output variable names, False for input variable names.
         """
-        super(PromotedToAbsoluteMap, self).__init__()
+        super(PromAbsDict, self).__init__()
 
         self._is_output = output
 
@@ -617,31 +798,37 @@ class PromotedToAbsoluteMap(dict):
                     abs_keys, prom_key = self._deriv_keys(key)
                     for abs_key in abs_keys:
                         self._values[abs_key] = values[key]
-                    super(PromotedToAbsoluteMap, self).__setitem__(prom_key, values[key])
+                    super(PromAbsDict, self).__setitem__(prom_key, values[key])
                 else:
                     if key in abs2prom:
                         # key is absolute name
                         self._values[key] = values[key]
                         prom_key = abs2prom[key]
-                        super(PromotedToAbsoluteMap, self).__setitem__(prom_key, values[key])
+                        super(PromAbsDict, self).__setitem__(prom_key, values[key])
                     elif key in prom2abs:
                         # key is promoted name
                         for abs_key in prom2abs[key]:
                             self._values[abs_key] = values[key]
-                        super(PromotedToAbsoluteMap, self).__setitem__(key, values[key])
+                        super(PromAbsDict, self).__setitem__(key, values[key])
             self._keys = self._values.keys()
         else:
             # numpy structured array, which will always use absolute names
-            self._values = values
+            self._values = values[0]
             self._keys = values.dtype.names
             for key in self._keys:
                 if key in abs2prom:
                     prom_key = abs2prom[key]
-                    super(PromotedToAbsoluteMap, self).__setitem__(prom_key, values[key])
+                    if prom_key in self:
+                        # We already set a value for this promoted name, which means
+                        # it is an input that maps to multiple absolute names. Set the
+                        # value to AMBIGOUS and require access via absolute name.
+                        super(PromAbsDict, self).__setitem__(prom_key, _AMBIGOUS_PROM_NAME)
+                    else:
+                        super(PromAbsDict, self).__setitem__(prom_key, self._values[key])
                 elif ',' in key:
                     # derivative keys will be a string in the form of 'of,wrt'
                     abs_keys, prom_key = self._deriv_keys(key)
-                    super(PromotedToAbsoluteMap, self).__setitem__(prom_key, values[key])
+                    super(PromAbsDict, self).__setitem__(prom_key, self._values[key])
 
     def __str__(self):
         """
@@ -652,7 +839,7 @@ class PromotedToAbsoluteMap(dict):
         str
             String representation of the dictionary.
         """
-        return super(PromotedToAbsoluteMap, self).__str__()
+        return super(PromAbsDict, self).__str__()
 
     def _deriv_keys(self, key):
         """
@@ -710,14 +897,21 @@ class PromotedToAbsoluteMap(dict):
 
         elif key in self:
             # promoted name
-            return super(PromotedToAbsoluteMap, self).__getitem__(key)
+            val = super(PromAbsDict, self).__getitem__(key)
+            if val is _AMBIGOUS_PROM_NAME:
+                msg = "The promoted name '%s' is invalid because it refers to multiple " + \
+                      "inputs: %s. Access the value using an absolute path name or the " + \
+                      "connected output variable instead."
+                raise RuntimeError(msg % (key, str(self._prom2abs['input'][key])))
+            else:
+                return val
 
         elif isinstance(key, tuple) or ',' in key:
             # derivative keys can be either (of, wrt) or 'of,wrt'
             abs_keys, prom_key = self._deriv_keys(key)
-            return super(PromotedToAbsoluteMap, self).__getitem__(prom_key)
+            return super(PromAbsDict, self).__getitem__(prom_key)
 
-        raise KeyError(key)
+        raise KeyError('Variable name "%s" not found.' % key)
 
     def __setitem__(self, key, value):
         """
@@ -744,18 +938,18 @@ class PromotedToAbsoluteMap(dict):
             for abs_key in abs_keys:
                 self._values[abs_key] = value
 
-            super(PromotedToAbsoluteMap, self).__setitem__(prom_key, value)
+            super(PromAbsDict, self).__setitem__(prom_key, value)
 
         elif key in self._keys:
             # absolute name
             self._values[key] = value
-            super(PromotedToAbsoluteMap, self).__setitem__(abs2prom[key], value)
+            super(PromAbsDict, self).__setitem__(abs2prom[key], value)
         else:
             # promoted name, propagate to all connected absolute names
             for abs_key in prom2abs[key]:
                 if abs_key in self._keys:
                     self._values[abs_key] = value
-            super(PromotedToAbsoluteMap, self).__setitem__(key, value)
+            super(PromAbsDict, self).__setitem__(key, value)
 
     def absolute_names(self):
         """
