@@ -16,6 +16,7 @@ from openmdao.test_suite.components.ae_tests import AEComp
 from openmdao.test_suite.components.sellar import SellarDerivatives, SellarDerivativesGrouped, \
     SellarProblem, SellarStateConnection, SellarProblemWithArrays
 from openmdao.test_suite.components.paraboloid import Paraboloid
+from openmdao.solvers.linesearch.tests.test_backtracking import ImplCompTwoStates
 
 from openmdao.recorders.tests.sqlite_recorder_test_utils import assertMetadataRecorded, \
     assertDriverIterDataRecorded, assertSystemIterDataRecorded, assertSolverIterDataRecorded, \
@@ -25,6 +26,7 @@ from openmdao.recorders.tests.sqlite_recorder_test_utils import assertMetadataRe
 from openmdao.recorders.tests.recorder_test_utils import run_driver
 from openmdao.utils.assert_utils import assert_rel_error
 from openmdao.utils.general_utils import determine_adder_scaler
+from openmdao.utils.testing_utils import use_tempdirs
 
 # check that pyoptsparse is installed. if it is, try to use SLSQP.
 OPT, OPTIMIZER = set_pyoptsparse_opt('SLSQP')
@@ -53,27 +55,15 @@ class ParaboloidProblem(om.Problem):
         model.add_constraint('c', upper=-15.0)
 
 
+@use_tempdirs
 class TestSqliteRecorder(unittest.TestCase):
 
     def setUp(self):
-        self.orig_dir = os.getcwd()
-        self.temp_dir = mkdtemp()
-        os.chdir(self.temp_dir)
-
-        self.filename = os.path.join(self.temp_dir, "sqlite_test")
+        self.filename = "sqlite_test"
         self.recorder = om.SqliteRecorder(self.filename, record_viewer_data=False)
 
         self.eps = 1e-3
 
-    def tearDown(self):
-        os.chdir(self.orig_dir)
-        try:
-            rmtree(self.temp_dir)
-        except OSError as e:
-            # If directory already deleted, keep going
-            if e.errno not in (errno.ENOENT, errno.EACCES, errno.EPERM):
-                raise e
-
     def test_only_desvars_recorded(self):
         prob = SellarProblem()
 
@@ -436,7 +426,7 @@ class TestSqliteRecorder(unittest.TestCase):
         assertMetadataRecorded(self, prom2abs, abs2prom)
         expected_problem_metadata = {
             'connections_list_length': 11,
-            'tree_length': 8,
+            'tree_length': 9,
             'tree_children_length': 7,
             'abs2prom': abs2prom,
         }
@@ -843,19 +833,24 @@ class TestSqliteRecorder(unittest.TestCase):
         assertSolverIterDataRecorded(self, expected_data, self.eps, prefix='run_again')
 
     def test_record_line_search_armijo_goldstein(self):
-        prob = SellarProblem()
+        prob = om.Problem()
+        prob.model.add_subsystem('px', om.IndepVarComp('x', 1.0))
+        prob.model.add_subsystem('comp', ImplCompTwoStates())
+        prob.model.connect('px.x', 'comp.x')
+
+        prob.model.nonlinear_solver = om.NewtonSolver()
+        prob.model.nonlinear_solver.options['maxiter'] = 10
+        prob.model.linear_solver = om.ScipyKrylov()
+
+        ls = prob.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
+        ls.add_recorder(self.recorder)
+        ls.options['c'] = .1
+
         prob.setup()
 
-        model = prob.model
-        model.linear_solver = om.ScipyKrylov()
-
-        nl = model.nonlinear_solver = om.NewtonSolver()
-        nl.options['solve_subsystems'] = True
-        nl.options['max_sub_solves'] = 4
-
-        ls = nl.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
-        ls.options['c'] = 100.0  # This is bogus, but it ensures that we get a few LS iterations.
-        ls.add_recorder(self.recorder)
+        prob['px.x'] = 2.0
+        prob['comp.y'] = 0.
+        prob['comp.z'] = 1.6
 
         t0, t1 = run_driver(prob)
 
@@ -869,18 +864,14 @@ class TestSqliteRecorder(unittest.TestCase):
             'ArmijoGoldsteinLS', (2,)
         ]
 
-        expected_abs_error = 5.6736837450444e-12
-        expected_rel_error = 0.0047475363051265665
+        expected_abs_error = 3.2882366094914777
+        expected_rel_error = 0.9999999999999998
 
         expected_solver_output = {
-            "con_cmp1.con1": [-22.42830237],
-            "d1.y1": [25.58830237],
-            "con_cmp2.con2": [-11.941511849],
-            "pz.z": [5.0, 2.0],
-            "obj_cmp.obj": [28.58830816516],
-            "d2.y2": [12.058488150],
-            "px.x": [1.0]
-        }
+            "comp.z": [1.5],
+            "comp.y": [1.75],
+            "px.x": [2.0],
+            }
 
         expected_solver_residuals = None
 
@@ -948,7 +939,6 @@ class TestSqliteRecorder(unittest.TestCase):
         nl.options['max_sub_solves'] = 4
 
         ls = nl.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
-        ls.options['c'] = 100.0  # This is bogus, but it ensures that we get a few LS iterations.
         model.add_recorder(self.recorder)
 
         try:
@@ -1109,7 +1099,7 @@ class TestSqliteRecorder(unittest.TestCase):
             try:
                 ln = nl.linear_solver = solver()
             except RuntimeError as err:
-                if str(err) == 'PETSc is not available.':
+                if str(err).endswith('PETSc is not available.'):
                     continue
                 else:
                     raise err
@@ -1738,25 +1728,8 @@ class TestSqliteRecorder(unittest.TestCase):
         self.assertAlmostEqual((unscaled_y + adder) * scaler, scaled_y, places=12)
 
 
+@use_tempdirs
 class TestFeatureSqliteRecorder(unittest.TestCase):
-    def setUp(self):
-        import os
-        from tempfile import mkdtemp
-        self.dir = mkdtemp()
-        self.original_path = os.getcwd()
-        os.chdir(self.dir)
-
-    def tearDown(self):
-        import os
-        import errno
-        from shutil import rmtree
-        os.chdir(self.original_path)
-        try:
-            rmtree(self.dir)
-        except OSError as e:
-            # If directory already deleted, keep going
-            if e.errno not in (errno.ENOENT, errno.EACCES, errno.EPERM):
-                raise e
 
     def test_feature_simple_driver_recording(self):
         import openmdao.api as om
@@ -1816,7 +1789,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
 
         # access list of connections stored in metadata
         connections = sorted(cr.problem_metadata['connections_list'], key=lambda x: (x['tgt'], x['src']))
-        
+
         self.assertEqual('\n'.join([conn['src']+'\t'+conn['tgt'] for conn in connections]),
                          '\n'.join(["d1.y1\tcon_cmp1.y1",
                                     "d2.y2\tcon_cmp2.y2",
@@ -1833,7 +1806,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         # access the model tree stored in metadata
         self.assertEqual(set(cr.problem_metadata['tree'].keys()),
                          {'name', 'type', 'subsystem_type', 'children', 'linear_solver',
-                          'nonlinear_solver', 'is_parallel', 'component_type'})
+                          'nonlinear_solver', 'is_parallel', 'component_type', 'class'})
         self.assertEqual(cr.problem_metadata['tree']['name'], 'root')
         self.assertEqual(sorted([child["name"] for child in cr.problem_metadata['tree']["children"]]),
                          ['con_cmp1', 'con_cmp2', 'd1', 'd2', 'obj_cmp', 'px', 'pz'])
@@ -1844,8 +1817,7 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
 
         prob = om.Problem(SellarDerivatives())
         model = prob.model
-        model.add_design_var('z', lower=np.array([-10.0, 0.0]),
-                                  upper=np.array([10.0, 10.0]))
+        model.add_design_var('z', lower=np.array([-10.0, 0.0]), upper=np.array([10.0, 10.0]))
         model.add_design_var('x', lower=0.0, upper=10.0)
         model.add_objective('obj')
         model.add_constraint('con1', upper=0.0)
@@ -2343,30 +2315,12 @@ class TestFeatureSqliteRecorder(unittest.TestCase):
         assert_rel_error(self, constraints, case.get_constraints(), 1e-1)
 
 
+@use_tempdirs
 class TestFeatureBasicRecording(unittest.TestCase):
+
     def setUp(self):
-        import os
-        from tempfile import mkdtemp
-
-        self.dir = mkdtemp()
-        self.original_path = os.getcwd()
-        os.chdir(self.dir)
-
         self.record_cases()
 
-    def tearDown(self):
-        import os
-        import errno
-        from shutil import rmtree
-
-        os.chdir(self.original_path)
-        try:
-            rmtree(self.dir)
-        except OSError as e:
-            # If directory already deleted, keep going
-            if e.errno not in (errno.ENOENT, errno.EACCES, errno.EPERM):
-                raise e
-
     def record_cases(self):
         import openmdao.api as om
         from openmdao.test_suite.components.sellar_feature import SellarMDA
