@@ -13,6 +13,12 @@ from openmdao.test_suite.components.expl_comp_simple import TestExplCompSimpleJa
 from openmdao.test_suite.components.sellar import SellarDerivatives
 from openmdao.test_suite.groups.implicit_group import TestImplicitGroup
 from openmdao.utils.assert_utils import assert_rel_error
+from openmdao.utils.mpi import MPI
+
+try:
+    from openmdao.vectors.petsc_vector import PETScVector
+except ImportError:
+    PETScVector = None
 
 
 class NanComp(om.ExplicitComponent):
@@ -88,9 +94,9 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         # Test that using options that should not exist in class cause an error
         solver = om.DirectSolver()
 
-        msg = "\"Option '%s' cannot be set because it has not been declared.\""
+        msg = "\"DirectSolver: Option '%s' cannot be set because it has not been declared.\""
 
-        for option in ['atol', 'rtol', 'maxiter', 'err_on_maxiter']:
+        for option in ['atol', 'rtol', 'maxiter', 'err_on_non_converge']:
             with self.assertRaises(KeyError) as context:
                 solver.options[option] = 1
 
@@ -220,11 +226,12 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
 
         prob.setup()
         prob.set_solver_print(level=0)
+        prob.final_setup()
 
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in 'thrust_equilibrium_group' for column associated with state/residual 'thrust'."
+        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -240,7 +247,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(Exception) as cm:
             prob.setup()
 
-        expected_msg = "dupcomp: declare_partials has been called with rows and cols that specify the following duplicate subjacobian entries: [(4, 11), (10, 2)]."
+        expected_msg = "DupPartialsComp (dupcomp): d(x)/d(c): declare_partials has been called with rows and cols that specify the following duplicate subjacobian entries: [(4, 11), (10, 2)]."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -278,7 +285,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in 'thrust_equilibrium_group' for column associated with state/residual 'thrust'."
+        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -315,7 +322,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.run_model()
 
-        expected_msg = "Singular entry found in 'thrust_equilibrium_group' for row associated with state/residual 'thrust'."
+        expected_msg = "Singular entry found in Group (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -381,7 +388,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in '' for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -414,7 +421,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in '' for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -447,7 +454,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in '' for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -476,7 +483,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "NaN entries found in '' for rows associated with states/residuals ['c5.y']."
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['c5.y']."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -505,7 +512,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "Singular entry found in '' for row associated with state/residual 'c5.y'."
+        expected_msg = "Singular entry found in Group (<model>) for row associated with state/residual 'c5.y' index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -535,7 +542,7 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
         with self.assertRaises(RuntimeError) as cm:
             prob.compute_totals(of=['c5.y'], wrt=['p.x'])
 
-        expected_msg = "Singular entry found in '' for column associated with state/residual 'c5.y'."
+        expected_msg = "Singular entry found in Group (<model>) for row/col associated with state/residual 'c5.y' index 0."
 
         self.assertEqual(expected_msg, str(cm.exception))
 
@@ -626,6 +633,114 @@ class TestDirectSolver(LinearSolverTests.LinearSolverTestCase):
             prob.run_model()
 
 
+@unittest.skipUnless(MPI and PETScVector, "MPI and PETSc are required.")
+class TestDirectSolverErrsMPI(unittest.TestCase):
+
+    N_PROCS = 2
+
+    def test_raise_error_on_singular(self):
+        prob = om.Problem()
+        model = prob.model
+
+        comp = om.IndepVarComp()
+        comp.add_output('dXdt:TAS', val=1.0)
+        comp.add_output('accel_target', val=2.0)
+        model.add_subsystem('des_vars', comp, promotes=['*'])
+
+        teg = model.add_subsystem('thrust_equilibrium_group', subsys=om.ParallelGroup())
+        teg.add_subsystem('dynamics', om.ExecComp('z = 2.0*thrust'), promotes=['*'])
+
+        thrust_bal = om.BalanceComp()
+        thrust_bal.add_balance(name='thrust', val=1207.1, lhs_name='dXdt:TAS',
+                               rhs_name='accel_target', eq_units='m/s**2', lower=-10.0, upper=10000.0)
+
+        teg.add_subsystem(name='thrust_bal', subsys=thrust_bal,
+                          promotes_inputs=['dXdt:TAS', 'accel_target'],
+                          promotes_outputs=['thrust'])
+
+        teg.linear_solver = om.DirectSolver()
+
+        teg.nonlinear_solver = om.NewtonSolver()
+        teg.nonlinear_solver.options['solve_subsystems'] = True
+        teg.nonlinear_solver.options['max_sub_solves'] = 1
+        teg.nonlinear_solver.options['atol'] = 1e-4
+
+        prob.setup()
+        prob.set_solver_print(level=0)
+
+        with self.assertRaises(RuntimeError) as cm:
+            prob.run_model()
+
+        expected_msg = "Singular entry found in ParallelGroup (thrust_equilibrium_group) for row associated with state/residual 'thrust' ('thrust_equilibrium_group.thrust_bal.thrust') index 0."
+
+        self.assertTrue(expected_msg in str(cm.exception), "Error msg doesn't contain '%s'" % expected_msg)
+
+    def test_raise_error_on_nan_sparse(self):
+
+        prob = om.Problem()
+        model = prob.model
+
+        model.add_subsystem('p', om.IndepVarComp('x', 2.0))
+        model.add_subsystem('c1', om.ExecComp('y = 4.0*x'))
+        sub = model.add_subsystem('sub', om.ParallelGroup())
+        sub.add_subsystem('c2', NanComp())
+        model.add_subsystem('c3', om.ExecComp('y = 4.0*x'))
+        model.add_subsystem('c4', NanComp2())
+        model.add_subsystem('c5', om.ExecComp('y = 3.0*x'))
+        model.add_subsystem('c6', om.ExecComp('y = 2.0*x'))
+
+        model.connect('p.x', 'c1.x')
+        model.connect('c1.y', 'sub.c2.x')
+        model.connect('sub.c2.y', 'c3.x')
+        model.connect('c3.y', 'c4.x')
+        model.connect('c4.y', 'c5.x')
+        model.connect('c4.y2', 'c6.x')
+
+        model.linear_solver = om.DirectSolver()
+
+        prob.setup()
+        prob.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            prob.compute_totals(of=['c5.y'], wrt=['p.x'])
+
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+
+        self.assertTrue(expected_msg in str(cm.exception), "NaN error message doesn't match '%s'" % expected_msg)
+
+    def test_raise_error_on_nan_dense(self):
+
+        prob = om.Problem(model=om.Group(assembled_jac_type='dense'))
+        model = prob.model
+
+        model.add_subsystem('p', om.IndepVarComp('x', 2.0))
+        model.add_subsystem('c1', om.ExecComp('y = 4.0*x'))
+        sub = model.add_subsystem('sub', om.ParallelGroup())
+        sub.add_subsystem('c2', NanComp())
+        model.add_subsystem('c3', om.ExecComp('y = 4.0*x'))
+        model.add_subsystem('c4', NanComp2())
+        model.add_subsystem('c5', om.ExecComp('y = 3.0*x'))
+        model.add_subsystem('c6', om.ExecComp('y = 2.0*x'))
+
+        model.connect('p.x', 'c1.x')
+        model.connect('c1.y', 'sub.c2.x')
+        model.connect('sub.c2.y', 'c3.x')
+        model.connect('c3.y', 'c4.x')
+        model.connect('c4.y', 'c5.x')
+        model.connect('c4.y2', 'c6.x')
+
+        model.linear_solver = om.DirectSolver(assemble_jac=True)
+
+        prob.setup()
+        prob.run_model()
+
+        with self.assertRaises(RuntimeError) as cm:
+            prob.compute_totals(of=['c5.y'], wrt=['p.x'])
+
+        expected_msg = "NaN entries found in Group (<model>) for rows associated with states/residuals ['sub.c2.y', 'c4.y']."
+        self.assertTrue(expected_msg in str(cm.exception), "NaN error message doesn't match '%s'" % expected_msg)
+
+
 class TestDirectSolverFeature(unittest.TestCase):
 
     def test_specify_solver(self):
