@@ -48,15 +48,6 @@ class LinearUserDefined(LinearSolver):
             Derivative mode, can be 'fwd' or 'rev'.
         rel_systems : set of str
             Set of names of relevant systems based on the current linear solve.
-
-        Returns
-        -------
-        boolean
-            Failure flag; True if failed to converge, False is successful.
-        float
-            absolute error.
-        float
-            relative error.
         """
         self._vec_names = vec_names
         self._rel_systems = rel_systems
@@ -78,6 +69,4 @@ class LinearUserDefined(LinearSolver):
 
             # run custom solver
             with system._unscaled_context(outputs=[d_outputs], residuals=[d_resids]):
-                fail, abs_error, rel_error = solve(d_outputs, d_resids, mode)
-
-        return fail, abs_error, rel_error
+                solve(d_outputs, d_resids, mode)
