@@ -1,4 +1,4 @@
-""" Test for the Backktracking Line Search"""
+""" Test for the Backtracking Line Search"""
 
 import sys
 import unittest
@@ -13,6 +13,7 @@ import openmdao.api as om
 from openmdao.test_suite.components.double_sellar import DoubleSellar
 from openmdao.test_suite.components.implicit_newton_linesearch \
     import ImplCompTwoStates, ImplCompTwoStatesArrays
+from openmdao.test_suite.components.sellar import SellarDis1, SellarDis2withDerivatives
 from openmdao.utils.assert_utils import assert_rel_error
 
 
@@ -100,7 +101,7 @@ class TestArmejoGoldsteinBounds(unittest.TestCase):
 
         ls = top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='scalar')
         ls.options['maxiter'] = 10
-        ls.options['alpha'] = 10.0
+        ls.options['alpha'] = 1.0
 
         # Setup again because we assigned a new linesearch
         top.setup()
@@ -110,14 +111,16 @@ class TestArmejoGoldsteinBounds(unittest.TestCase):
         top['comp.y'] = 0.0
         top['comp.z'] = 1.6
         top.run_model()
-        self.assertTrue(1.5 <= top['comp.z'] <= 1.6)
+        self.assertGreaterEqual(top['comp.z'], 1.5)
+        self.assertLessEqual(top['comp.z'], 1.6)
 
         # Test lower bound: should stop just short of the upper bound
         top['px.x'] = 0.5
         top['comp.y'] = 0.0
         top['comp.z'] = 2.4
         top.run_model()
-        self.assertTrue(2.4 <= top['comp.z'] <= 2.5)
+        self.assertGreaterEqual(top['comp.z'], 2.5)
+        self.assertLessEqual(top['comp.z'], 2.5)
 
 
 class ParaboloidAE(om.ExplicitComponent):
@@ -283,7 +286,7 @@ class TestAnalysisErrorImplicit(unittest.TestCase):
         ls.options['maxiter'] = 5
         ls.options['alpha'] = 10.0
         ls.options['retry_on_analysis_error'] = True
-        ls.options['c'] = 10000.0
+        ls.options['c'] = 1.0
 
         top.setup()
         top.set_solver_print(level=2)
@@ -332,7 +335,7 @@ class TestAnalysisErrorImplicit(unittest.TestCase):
         ls.options['maxiter'] = 5
         ls.options['alpha'] = 10.0
         ls.options['retry_on_analysis_error'] = True
-        ls.options['c'] = 10000.0
+        ls.options['c'] = 1.0
 
         top.setup()
         top.set_solver_print(level=2)
@@ -451,7 +454,8 @@ class TestBoundsEnforceLSArrayBounds(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
         for ind in range(3):
-            self.assertTrue(1.5 <= top['comp.z'][ind] <= 1.6)
+            self.assertGreaterEqual(top['comp.z'][ind], 1.5)
+            self.assertLessEqual(top['comp.z'][ind], 1.6)
 
         # Test upper bounds: should stop just short of the minimum upper bound
         top['px.x'] = 0.5
@@ -513,33 +517,73 @@ class TestBoundsEnforceLSArrayBounds(unittest.TestCase):
     def test_undeclared_options(self):
         # Test that using options that should not exist in class, cause an
         # error if they are set when instantiating BoundsEnforceLS.
-        # atol, rtol, maxiter, and err_on_maxiter are not used in BoundsEnforceLS
+        # atol, rtol, maxiter, and err_on_non_converge are not used in BoundsEnforceLS
 
         with self.assertRaises(KeyError) as context:
             om.BoundsEnforceLS(bound_enforcement='scalar', atol=1.0)
 
-        self.assertEqual(str(context.exception), "\"Option 'atol' cannot be set because it "
+        self.assertEqual(str(context.exception), "\"BoundsEnforceLS: Option 'atol' cannot be set because it "
                                                  "has not been declared.\"")
 
         with self.assertRaises(KeyError) as context:
             om.BoundsEnforceLS(bound_enforcement='scalar', rtol=2.0)
 
-        self.assertEqual(str(context.exception), "\"Option 'rtol' cannot be set because it "
+        self.assertEqual(str(context.exception), "\"BoundsEnforceLS: Option 'rtol' cannot be set because it "
                                                  "has not been declared.\"")
 
         with self.assertRaises(KeyError) as context:
             om.BoundsEnforceLS(bound_enforcement='scalar', maxiter=1)
 
-        self.assertEqual(str(context.exception), "\"Option 'maxiter' cannot be set because it "
+        self.assertEqual(str(context.exception), "\"BoundsEnforceLS: Option 'maxiter' cannot be set because it "
                                                  "has not been declared.\"")
 
         with self.assertRaises(KeyError) as context:
-            om.BoundsEnforceLS(bound_enforcement='scalar', err_on_maxiter=True)
+            om.BoundsEnforceLS(bound_enforcement='scalar', err_on_non_converge=True)
 
-        self.assertEqual(str(context.exception), "\"Option 'err_on_maxiter' cannot be set because it "
+        self.assertEqual(str(context.exception), "\"BoundsEnforceLS: Option 'err_on_non_converge' cannot be set because it "
                                                  "has not been declared.\"")
 
 
+class SellarDis1withDerivativesMod(SellarDis1):
+    # Version of Sellar discipline 1 with a slightly incorrect x derivative.
+    # This will still solve, but will require some backtracking at times.
+
+    def _do_declares(self):
+        self.declare_partials(of='*', wrt='*')
+
+    def compute_partials(self, inputs, partials):
+        partials['y1', 'y2'] = -0.2
+        partials['y1', 'z'] = np.array([[2.0 * inputs['z'][0], 1.0]])
+        partials['y1', 'x'] = 1.5
+
+
+class SubSellarMod(om.Group):
+
+    def __init__(self, units=None, scaling=None, **kwargs):
+        super(SubSellarMod, self).__init__(**kwargs)
+
+        self.add_subsystem('d1', SellarDis1withDerivativesMod(units=units, scaling=scaling),
+                           promotes=['x', 'z', 'y1', 'y2'])
+        self.add_subsystem('d2', SellarDis2withDerivatives(units=units, scaling=scaling),
+                           promotes=['z', 'y1', 'y2'])
+
+
+class DoubleSellarMod(om.Group):
+
+    def __init__(self, units=None, scaling=None, **kwargs):
+        super(DoubleSellarMod, self).__init__(**kwargs)
+
+        self.add_subsystem('g1', SubSellarMod(units=units, scaling=scaling))
+        self.add_subsystem('g2', SubSellarMod(units=units, scaling=scaling))
+
+        self.connect('g1.y2', 'g2.x')
+        self.connect('g2.y2', 'g1.x')
+
+        # Converge the outer loop with Gauss Seidel, with a looser tolerance.
+        self.nonlinear_solver = om.NewtonSolver()
+        self.linear_solver = om.DirectSolver()
+
+
 class TestArmijoGoldsteinLSArrayBounds(unittest.TestCase):
 
     def setUp(self):
@@ -632,8 +676,9 @@ class TestArmijoGoldsteinLSArrayBounds(unittest.TestCase):
             self.assertTrue(2.4 <= top['comp.z'][ind] <= self.ub[ind])
 
     def test_with_subsolves(self):
+
         prob = om.Problem()
-        model = prob.model = DoubleSellar()
+        model = prob.model = DoubleSellarMod()
 
         g1 = model.g1
         g1.nonlinear_solver = om.NewtonSolver()
@@ -652,9 +697,6 @@ class TestArmijoGoldsteinLSArrayBounds(unittest.TestCase):
         model.nonlinear_solver.options['max_sub_solves'] = 4
         ls = model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
 
-        # This is pretty bogus, but it ensures that we get a few LS iterations.
-        ls.options['c'] = 100.0
-
         prob.set_solver_print(level=0)
 
         prob.setup()
@@ -760,9 +802,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_armijogoldsteinls_basic(self):
         import numpy as np
@@ -789,9 +830,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_boundscheck_basic(self):
         import numpy as np
@@ -818,9 +858,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_boundscheck_vector(self):
         import numpy as np
@@ -837,8 +876,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
-        ls = top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='vector')
-        ls.options['bound_enforcement'] = 'vector'
+        top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='vector')
 
         top.setup()
 
@@ -848,9 +886,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_boundscheck_wall(self):
         import numpy as np
@@ -867,8 +904,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
-        ls = top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='wall')
-        ls.options['bound_enforcement'] = 'wall'
+        top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='wall')
 
         top.setup()
 
@@ -897,8 +933,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
-        ls = top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='scalar')
-        ls.options['bound_enforcement'] = 'scalar'
+        top.model.nonlinear_solver.linesearch = om.BoundsEnforceLS(bound_enforcement='scalar')
 
         top.setup()
         top.run_model()
@@ -909,10 +944,6 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        print(top['comp.z'][0])
-        print(top['comp.z'][1])
-        print(top['comp.z'][2])
-
     def test_feature_print_bound_enforce(self):
         import numpy as np
 
@@ -940,9 +971,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_armijo_boundscheck_vector(self):
         import numpy as np
@@ -959,8 +989,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
-        ls = top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
-        ls.options['bound_enforcement'] = 'vector'
+        top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
 
         top.setup()
 
@@ -970,9 +999,8 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
     def test_feature_armijo_boundscheck_wall(self):
         import numpy as np
@@ -989,8 +1017,7 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.nonlinear_solver.options['maxiter'] = 10
         top.model.linear_solver = om.ScipyKrylov()
 
-        ls = top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='wall')
-        ls.options['bound_enforcement'] = 'wall'
+        top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='wall')
 
         top.setup()
 
@@ -1020,7 +1047,6 @@ class TestFeatureLineSearch(unittest.TestCase):
         top.model.linear_solver = om.ScipyKrylov()
 
         ls = top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='scalar')
-        ls.options['bound_enforcement'] = 'scalar'
 
         top.setup()
         top.run_model()
@@ -1031,10 +1057,6 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        print(top['comp.z'][0])
-        print(top['comp.z'][1])
-        print(top['comp.z'][2])
-
     def test_feature_armijo_print_bound_enforce(self):
         import numpy as np
 
@@ -1062,9 +1084,37 @@ class TestFeatureLineSearch(unittest.TestCase):
         top['comp.z'] = 1.6
         top.run_model()
 
-        assert_rel_error(self, top['comp.z'][0], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][1], [1.5], 1e-8)
-        assert_rel_error(self, top['comp.z'][2], [1.5], 1e-8)
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
+
+    def test_feature_goldstein(self):
+        import numpy as np
+
+        import openmdao.api as om
+        from openmdao.test_suite.components.implicit_newton_linesearch import ImplCompTwoStatesArrays
+
+        top = om.Problem()
+        top.model.add_subsystem('px', om.IndepVarComp('x', np.ones((3, 1))))
+        top.model.add_subsystem('comp', ImplCompTwoStatesArrays())
+        top.model.connect('px.x', 'comp.x')
+
+        top.model.nonlinear_solver = om.NewtonSolver()
+        top.model.nonlinear_solver.options['maxiter'] = 10
+        top.model.linear_solver = om.ScipyKrylov()
+
+        ls = top.model.nonlinear_solver.linesearch = om.ArmijoGoldsteinLS(bound_enforcement='vector')
+        ls.options['method'] = 'Goldstein'
+
+        top.setup()
+
+        # Test lower bounds: should go to the lower bound and stall
+        top['px.x'] = 2.0
+        top['comp.y'] = 0.
+        top['comp.z'] = 1.6
+        top.run_model()
+
+        for ind in range(3):
+            assert_rel_error(self, top['comp.z'][ind], [1.5], 1e-8)
 
 
 if __name__ == "__main__":
