@@ -149,8 +149,9 @@ def l1_cross_distances(X, Y=None):
         The array of componentwise L1 cross-distances.
 
     """
+    X = array2d(X)
+
     if Y is None:
-        X = array2d(X)
         n_samples, n_features = X.shape
         n_nonzero_cross_dist = n_samples * (n_samples - 1) // 2
         D = np.zeros((n_nonzero_cross_dist, n_features))
@@ -160,7 +161,6 @@ def l1_cross_distances(X, Y=None):
             ll_1 = ll_0 + n_samples - k - 1
             D[ll_0:ll_1] = np.abs(X[k] - X[(k + 1):])
     else:
-        X = array2d(X)
         Y = array2d(Y)
         n_samples_X, n_features_X = X.shape
         n_samples_Y, n_features_Y = Y.shape
@@ -187,6 +187,12 @@ class MultiFiCoKriging(object):
     ----------
     corr : Object
         Correlation function to use, default is squared_exponential_correlation.
+    n_features : ndarry
+        Number of features for each fidelity level.
+    n_samples : ndarry
+        Number of samples for each fidelity level.
+    nlevel : int
+        Number of fidelity levels.
     normalize : bool, optional
         When true, normalize X and Y so that the mean is at zero.
     regr : string or callable
@@ -366,6 +372,10 @@ class MultiFiCoKriging(object):
         self.y_mean = 0
         self.y_std = 1
 
+        self.n_features = None
+        self.n_samples = None
+        self.nlevel = None
+
         self._nfev = 0
 
     def _build_R(self, lvl, theta):
@@ -459,8 +469,8 @@ class MultiFiCoKriging(object):
             # Calculate matrix of distances D between samples
             self.D[lvl] = l1_cross_distances(X[lvl])
             if (np.min(np.sum(self.D[lvl], axis=1)) == 0.):
-                raise Exception("Multiple input features cannot have the same"
-                                " value.")
+                raise ValueError("Multiple input features cannot have the same"
+                                 " value.")
 
             # Regression matrix and parameters
             self.F[lvl] = self.regr(X[lvl])
@@ -478,15 +488,15 @@ class MultiFiCoKriging(object):
             n_samples_F_i = self.F[lvl].shape[0]
 
             if n_samples_F_i != n_samples[lvl]:
-                raise Exception("Number of rows in F and X do not match. Most "
-                                "likely something is going wrong with the "
-                                "regression model.")
+                raise ValueError("Number of rows in F and X do not match. Most "
+                                 "likely something is going wrong with the "
+                                 "regression model.")
 
             if int(self.p[lvl] + self.q[lvl]) >= n_samples_F_i:
-                raise Exception(("Ordinary least squares problem is undetermined "
-                                 "n_samples=%d must be greater than the regression"
-                                 " model size p+q=%d.")
-                                % (n_samples[i], self.p[lvl] + self.q[lvl]))
+                raise ValueError(("Ordinary least squares problem is undetermined "
+                                  "n_samples=%d must be greater than the regression"
+                                  " model size p+q=%d.")
+                                 % (n_samples[lvl], self.p[lvl] + self.q[lvl]))
 
         # Set attributes
         self.X = X
@@ -498,18 +508,17 @@ class MultiFiCoKriging(object):
             # Determine Gaussian Process model parameters
             if self.theta[lvl] is None:
                 # Maximum Likelihood Estimation of the parameters
-                sol = self._max_rlf(
-                    lvl=lvl, initial_range=initial_range, tol=tol)
+                sol = self._max_rlf(lvl=lvl, initial_range=initial_range, tol=tol)
                 self.theta[lvl] = sol['theta']
                 self.rlf_value[lvl] = sol['rlf_value']
 
                 if np.isinf(self.rlf_value[lvl]):
-                    raise Exception("Bad parameter region. "
-                                    "Try increasing upper bound")
+                    raise ValueError("Bad parameter region. "
+                                     "Try increasing upper bound")
             else:
                 self.rlf_value[lvl] = self.rlf(lvl=lvl)
                 if np.isinf(self.rlf_value[lvl]):
-                    raise Exception("Bad point. Try increasing theta0.")
+                    raise ValueError("Bad point. Try increasing theta0.")
 
         return
 
@@ -708,7 +717,6 @@ class MultiFiCoKriging(object):
         mu[:, 0] = (np.dot(f, beta) + np.dot(r_, gamma)).ravel()
 
         if eval_MSE:
-            self.sigma2_rho = nlevel * [None]
             MSE = np.zeros((n_eval, nlevel))
             r_t = solve_triangular(C, r_.T, lower=True)
             G = self.G[0]
@@ -794,7 +802,7 @@ class MultiFiCoKriging(object):
         n_samples_y = np.zeros(nlevel, dtype=int)
         for i in range(nlevel):
             n_samples[i], n_features[i] = X[i].shape
-            if i > 1 and n_features[i] != n_features[i - 1]:
+            if i > 0 and n_features[i] != n_features[i - 1]:
                 raise ValueError("All X must have the same number of columns.")
             y[i] = np.asarray(y[i]).ravel()[:, np.newaxis]
             n_samples_y[i] = y[i].shape[0]
@@ -811,17 +819,17 @@ class MultiFiCoKriging(object):
         if type(self.theta0) is not list:
             self.theta0 = nlevel * [self.theta0]
         elif len(self.theta0) != nlevel:
-            raise ValueError("theta0 must be a list of %d elements." % nlevel)
+            raise ValueError("theta0 must be a list of %d element(s)." % nlevel)
 
         if type(self.thetaL) is not list:
             self.thetaL = nlevel * [self.thetaL]
         elif len(self.thetaL) != nlevel:
-            raise ValueError("thetaL must be a list of %d elements." % nlevel)
+            raise ValueError("thetaL must be a list of %d element(s)." % nlevel)
 
         if type(self.thetaU) is not list:
             self.thetaU = nlevel * [self.thetaU]
         elif len(self.thetaU) != nlevel:
-            raise ValueError("thetaU must be a list of %d elements." % nlevel)
+            raise ValueError("thetaU must be a list of %d element(s)." % nlevel)
 
         self.nlevel = nlevel
         self.X = X[:]
@@ -857,7 +865,7 @@ class MultiFiCoKriging(object):
             if self.theta[i] is not None:
                 self.theta[i] = array2d(self.theta[i])
                 if np.any(self.theta[i] <= 0):
-                    raise ValueError("theta0 must be strictly positive.")
+                    raise ValueError("theta must be strictly positive.")
 
             if self.theta0[i] is not None:
                 self.theta0[i] = array2d(self.theta0[i])
@@ -899,82 +907,77 @@ class MultiFiCoKrigingSurrogate(MultiFiSurrogateModel):
 
     Attributes
     ----------
-    initial_range : float
-        Initial range for the optimizer.
     model : MultiFiCoKriging
         Contains MultiFiCoKriging surrogate.
-    tolerance : float
-        Optimizer terminates when the tolerance tol is reached.
     """
 
-    def __init__(self, regr='constant', rho_regr='constant', normalize=True,
-                 theta=None, theta0=None, thetaL=None, thetaU=None,
-                 tolerance=TOLERANCE_DEFAULT, initial_range=INITIAL_RANGE_DEFAULT):
+    def __init__(self, **kwargs):
         """
         Initialize all attributes.
 
         Parameters
         ----------
-        normalize : bool, optional
-            When true, normalize X and Y so that the mean is at zero.
-        regr : string or callable, optional
-            A regression function returning an array of outputs of the linear
-            regression functional basis for Universal Kriging purpose.
-            regr is assumed to be the same for all levels of code.
-            Default assumes a simple constant regression trend.
-            Available built-in regression models are:
-            'constant', 'linear'
-        rho_regr : string or callable, optional
-            A regression function returning an array of outputs of the linear
-            regression functional basis. Defines the regression function for the
-            autoregressive parameter rho.
-            rho_regr is assumed to be the same for all levels of code.
-            Default assumes a simple constant regression trend.
-            Available built-in regression models are:
-            'constant', 'linear'
-        theta : double, array_like or list, optional
-            Value of correlation parameters if they are known; no optimization is run.
-            Default is None, so that optimization is run.
-            if double: value is replicated for all features and all levels.
-            if array_like: an array with shape (n_features, ) for
-            isotropic calculation. It is replicated for all levels.
-            if list: a list of nlevel arrays specifying value for each level
-        theta0 : double, array_like or list, optional
-            Starting point for the maximum likelihood estimation of the
-            best set of parameters.
-            Default is None and meaning use of the default 0.5*np.ones(n_features)
-            if double: value is replicated for all features and all levels.
-            if array_like: an array with shape (n_features, ) for
-            isotropic calculation. It is replicated for all levels.
-            if list: a list of nlevel arrays specifying value for each level
-        thetaL : double, array_like or list, optional
-            Lower bound on the autocorrelation parameters for maximum
-            likelihood estimation.
-            Default is None meaning use of the default 1e-5*np.ones(n_features).
-            if double: value is replicated for all features and all levels.
-            if array_like: An array with shape matching theta0's. It is replicated
-            for all levels of code.
-            if list: a list of nlevel arrays specifying value for each level
-        thetaU : double, array_like or list, optional
-            Upper bound on the autocorrelation parameters for maximum
-            likelihood estimation.
-            Default is None meaning use of default value 50*np.ones(n_features).
-            if double: value is replicated for all features and all levels.
-            if array_like: An array with shape matching theta0's. It is replicated
-            for all levels of code.
-            if list: a list of nlevel arrays specifying value for each level
-        tolerance : float
-            Optimizer terminates when the tolerance tol is reached.
-        initial_range : float
-            Initial range for the optimizer.
+        **kwargs : keyword args
+            Some implementations of record_derivatives need additional args.
         """
-        super(MultiFiCoKrigingSurrogate, self).__init__()
+        super(MultiFiCoKrigingSurrogate, self).__init__(**kwargs)
+        self.model = None
 
-        self.tolerance = tolerance
-        self.initial_range = initial_range
-        self.model = MultiFiCoKriging(regr=regr, rho_regr=rho_regr, theta=theta,
-                                      theta0=theta0, thetaL=thetaL, thetaU=thetaU,
-                                      normalize=normalize)
+    def _declare_options(self):
+        """
+        Declare options before kwargs are processed in the init method.
+        """
+        opt = self.options
+        opt.declare('normalize', default=True, types=bool,
+                    desc="When true, normalize X and Y so that the mean is at zero.")
+        opt.declare('regr', default='constant', types=(object, ),
+                    desc="A regression function returning an array of outputs of the linear "
+                    "regression functional basis for Universal Kriging purpose. regr is assumed "
+                    "to be the same for all levels of code. Default assumes a simple constant "
+                    "regression trend. Available built-in regression models can be accessed by "
+                    "setting this option to the strings 'constant' or 'linear'")
+        opt.declare('rho_regr', default='constant', types=(object, ),
+                    desc="A regression function returning an array of outputs of the linear "
+                    "regression functional basis. Defines the regression function for the "
+                    "autoregressive parameter rho.. regr is assumed to be the same for all levels "
+                    "of code. Default assumes a simple constant regression trend. Available "
+                    "built-in regression models can be accessed by setting this option to the "
+                    "strings 'constant' or 'linear'")
+        opt.declare('theta', default=None, allow_none=True,
+                    desc="Value of correlation parameters. If they are known, then no "
+                    "optimization is run. Default is None, so that optimization is run. if double, "
+                    "then value is replicated for all features and all levels. if array_like, "
+                    "then an array with shape (n_features, ) for isotropic calculation. It is "
+                    "replicated for all levels. if list, then a list of nlevel arrays specifying "
+                    "value for each level")
+        opt.declare('theta0', default=None, allow_none=True,
+                    desc="Starting point for the maximum likelihood estimation of the best set "
+                    "of parameters. "
+                    "Default is None and meaning use of the default 0.5*np.ones(n_features) "
+                    "if double: value is replicated for all features and all levels. "
+                    "if array_like: an array with shape (n_features, ) for "
+                    "isotropic calculation. It is replicated for all levels. "
+                    "if list: a list of nlevel arrays specifying value for each level")
+        opt.declare('thetaL', default=None, allow_none=True,
+                    desc="Lower bound on the autocorrelation parameters for maximum "
+                    "likelihood estimation."
+                    "Default is None meaning use of the default 1e-5*np.ones(n_features). "
+                    "if double: value is replicated for all features and all levels. "
+                    "if array_like: An array with shape matching theta0s. It is replicate "
+                    "for all levels of code. "
+                    "if list: a list of nlevel arrays specifying value for each level")
+        opt.declare('thetaU', default=None, allow_none=True,
+                    desc="Upper bound on the autocorrelation parameters for maximum "
+                    "likelihood estimation. "
+                    "Default is None meaning use of default value 50*np.ones(n_features). "
+                    "if double: value is replicated for all features and all levels. "
+                    "if array_like: An array with shape matching theta0's. It is replicated "
+                    "for all levels of code. "
+                    "if list: a list of nlevel arrays specifying value for each level")
+        opt.declare('tolerance', default=TOLERANCE_DEFAULT,
+                    desc='Optimizer terminates when the tolerance tol is reached.')
+        opt.declare('initial_range', default=INITIAL_RANGE_DEFAULT,
+                    desc='Initial range for the optimizer.')
 
     def predict(self, new_x):
         """
@@ -1012,9 +1015,16 @@ class MultiFiCoKrigingSurrogate(MultiFiSurrogateModel):
             An array with shape (n_samples_X, n_features) with the observations of the scalar
             output to be predicted.
         """
+        opt = self.options
+        if not self.model:
+            self.model = MultiFiCoKriging(regr=opt['regr'], rho_regr=opt['rho_regr'],
+                                          theta=opt['theta'], theta0=opt['theta0'],
+                                          thetaL=opt['thetaL'], thetaU=opt['thetaU'],
+                                          normalize=opt['normalize'])
+
         X, Y = self._fit_adapter(X, Y)
-        self.model.fit(X, Y, tol=self.tolerance,
-                       initial_range=self.initial_range)
+        self.model.fit(X, Y, tol=opt['tolerance'],
+                       initial_range=opt['initial_range'])
 
     def _fit_adapter(self, X, Y):
         """
@@ -1047,8 +1057,3 @@ class MultiFiCoKrigingSurrogate(MultiFiSurrogateModel):
         X = [np.array(x) for x in reversed(X)]
         Y = [np.array(y) for y in reversed(Y)]
         return (X, Y)
-
-
-if __name__ == "__main__":
-    import doctest
-    doctest.testmod()
