@@ -2,11 +2,11 @@
 Some classes that are used to test Analysis Errors on multiple processes.
 """
 
-from openmdao.api import ExplicitComponent, AnalysisError
+import openmdao.api as om
 from openmdao.core.driver import Driver
 
 
-class AEComp(ExplicitComponent):
+class AEComp(om.ExplicitComponent):
 
     def setup(self):
         self.add_input('x', val=0.0)
@@ -19,10 +19,11 @@ class AEComp(ExplicitComponent):
         x = inputs['x']
 
         if x > 2.0:
-            raise AnalysisError('Try again.')
+            raise om.AnalysisError('Try again.')
 
         outputs['y'] = x*x + 2.0
 
+
 class AEDriver(Driver):
     """
     Handle an Analysis Error from below.
@@ -34,7 +35,7 @@ class AEDriver(Driver):
         """
         try:
             self._problem.model.run_solve_nonlinear()
-        except AnalysisError:
+        except om.AnalysisError:
             return True
 
         return False
\ No newline at end of file
