@@ -7,15 +7,15 @@ From Sellar's analytic problem.
     Optimization for Multidisciplinary System Design," Proceedings References 79 of the 34th AIAA
     Aerospace Sciences Meeting and Exhibit, Reno, NV, January 1996.
 """
-
 import numpy as np
-from openmdao.api import Group, ExplicitComponent, ExecComp, IndepVarComp, \
-                         NonlinearBlockGS, ScipyKrylov, DirectSolver, EQConstraintComp, NewtonSolver
+
+import openmdao.api as om
 from openmdao.test_suite.components.sellar import SellarDis1withDerivatives, \
                          SellarDis2withDerivatives
 from openmdao.test_suite.components.double_sellar import SubSellar
 
-class SellarDis1(ExplicitComponent):
+
+class SellarDis1(om.ExplicitComponent):
     """
     Component containing Discipline 1 -- no derivatives version.
     """
@@ -50,7 +50,7 @@ class SellarDis1(ExplicitComponent):
         outputs['y1'] = z1**2 + z2 + x1 - 0.2*y2
 
 
-class SellarDis2(ExplicitComponent):
+class SellarDis2(om.ExplicitComponent):
     """
     Component containing Discipline 2 -- no derivatives version.
     """
@@ -87,57 +87,61 @@ class SellarDis2(ExplicitComponent):
         outputs['y2'] = y1**.5 + z1 + z2
 
 
-class SellarMDA(Group):
+class SellarMDA(om.Group):
     """
     Group containing the Sellar MDA.
     """
 
     def setup(self):
-        indeps = self.add_subsystem('indeps', IndepVarComp(), promotes=['*'])
+        indeps = self.add_subsystem('indeps', om.IndepVarComp(), promotes=['*'])
         indeps.add_output('x', 1.0)
         indeps.add_output('z', np.array([5.0, 2.0]))
 
-        cycle = self.add_subsystem('cycle', Group(), promotes=['*'])
-        cycle.add_subsystem('d1', SellarDis1(), promotes_inputs=['x', 'z', 'y2'], promotes_outputs=['y1'])
-        cycle.add_subsystem('d2', SellarDis2(), promotes_inputs=['z', 'y1'], promotes_outputs=['y2'])
+        cycle = self.add_subsystem('cycle', om.Group(), promotes=['*'])
+        cycle.add_subsystem('d1', SellarDis1(), promotes_inputs=['x', 'z', 'y2'],
+                            promotes_outputs=['y1'])
+        cycle.add_subsystem('d2', SellarDis2(), promotes_inputs=['z', 'y1'],
+                            promotes_outputs=['y2'])
 
         # Nonlinear Block Gauss Seidel is a gradient free solver
-        cycle.nonlinear_solver = NonlinearBlockGS()
+        cycle.nonlinear_solver = om.NonlinearBlockGS()
 
-        self.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
-                           z=np.array([0.0, 0.0]), x=0.0),
+        self.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                  z=np.array([0.0, 0.0]), x=0.0),
                            promotes=['x', 'z', 'y1', 'y2', 'obj'])
 
-        self.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
-        self.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+        self.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        self.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
 
-class SellarMDALinearSolver(Group):
+class SellarMDALinearSolver(om.Group):
     """
     Group containing the Sellar MDA.
     """
 
     def setup(self):
-        indeps = self.add_subsystem('indeps', IndepVarComp(), promotes=['*'])
+        indeps = self.add_subsystem('indeps', om.IndepVarComp(), promotes=['*'])
         indeps.add_output('x', 1.0)
         indeps.add_output('z', np.array([5.0, 2.0]))
 
-        cycle = self.add_subsystem('cycle', Group(), promotes=['*'])
-        d1 = cycle.add_subsystem('d1', SellarDis1(), promotes_inputs=['x', 'z', 'y2'], promotes_outputs=['y1'])
-        d2 = cycle.add_subsystem('d2', SellarDis2(), promotes_inputs=['z', 'y1'], promotes_outputs=['y2'])
+        cycle = self.add_subsystem('cycle', om.Group(), promotes=['*'])
+        d1 = cycle.add_subsystem('d1', SellarDis1(), promotes_inputs=['x', 'z', 'y2'],
+                                 promotes_outputs=['y1'])
+        d2 = cycle.add_subsystem('d2', SellarDis2(), promotes_inputs=['z', 'y1'],
+                                 promotes_outputs=['y2'])
 
-        cycle.nonlinear_solver = NonlinearBlockGS()
-        cycle.linear_solver = DirectSolver()
+        cycle.nonlinear_solver = om.NonlinearBlockGS()
+        cycle.linear_solver = om.DirectSolver()
 
-        self.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+        self.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                            z=np.array([0.0, 0.0]), x=0.0),
                            promotes=['x', 'z', 'y1', 'y2', 'obj'])
 
-        self.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
-        self.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+        self.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        self.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
 
-class SellarDis1CS(ExplicitComponent):
+class SellarDis1CS(om.ExplicitComponent):
     """
     Component containing Discipline 1 -- no derivatives version.
     Uses Complex Step
@@ -173,7 +177,7 @@ class SellarDis1CS(ExplicitComponent):
         outputs['y1'] = z1**2 + z2 + x1 - 0.2*y2
 
 
-class SellarDis2CS(ExplicitComponent):
+class SellarDis2CS(om.ExplicitComponent):
     """
     Component containing Discipline 2 -- no derivatives version.
     Uses Complex Step
@@ -211,37 +215,37 @@ class SellarDis2CS(ExplicitComponent):
         outputs['y2'] = y1**.5 + z1 + z2
 
 
-class SellarNoDerivativesCS(Group):
+class SellarNoDerivativesCS(om.Group):
     """
     Group containing the Sellar MDA. This version uses the disciplines without derivatives.
     """
 
     def setup(self):
-        self.add_subsystem('px', IndepVarComp('x', 1.0), promotes=['x'])
-        self.add_subsystem('pz', IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
+        self.add_subsystem('px', om.IndepVarComp('x', 1.0), promotes=['x'])
+        self.add_subsystem('pz', om.IndepVarComp('z', np.array([5.0, 2.0])), promotes=['z'])
 
-        cycle = self.add_subsystem('cycle', Group(), promotes=['x', 'z', 'y1', 'y2'])
+        cycle = self.add_subsystem('cycle', om.Group(), promotes=['x', 'z', 'y1', 'y2'])
         d1 = cycle.add_subsystem('d1', SellarDis1CS(), promotes=['x', 'z', 'y1', 'y2'])
         d2 = cycle.add_subsystem('d2', SellarDis2CS(), promotes=['z', 'y1', 'y2'])
 
-        self.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
-                           z=np.array([0.0, 0.0]), x=0.0),
+        self.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+                                                  z=np.array([0.0, 0.0]), x=0.0),
                            promotes=['x', 'z', 'y1', 'y2', 'obj'])
 
-        self.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
-        self.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
+        self.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'), promotes=['con1', 'y1'])
+        self.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'), promotes=['con2', 'y2'])
 
-        self.nonlinear_solver = NonlinearBlockGS()
-        self.linear_solver = ScipyKrylov()
+        self.nonlinear_solver = om.NonlinearBlockGS()
+        self.linear_solver = om.ScipyKrylov()
 
 
-class SellarIDF(Group):
+class SellarIDF(om.Group):
     """
     Individual Design Feasible (IDF) architecture for the Sellar problem.
     """
     def setup(self):
         # construct the Sellar model with `y1` and `y2` as independent variables
-        dv = IndepVarComp()
+        dv = om.IndepVarComp()
         dv.add_output('x', 5.)
         dv.add_output('y1', 5.)
         dv.add_output('y2', 5.)
@@ -251,11 +255,11 @@ class SellarIDF(Group):
         self.add_subsystem('d1', SellarDis1withDerivatives())
         self.add_subsystem('d2', SellarDis2withDerivatives())
 
-        self.add_subsystem('obj_cmp', ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
+        self.add_subsystem('obj_cmp', om.ExecComp('obj = x**2 + z[1] + y1 + exp(-y2)',
                            x=0., z=np.array([0., 0.])))
 
-        self.add_subsystem('con_cmp1', ExecComp('con1 = 3.16 - y1'))
-        self.add_subsystem('con_cmp2', ExecComp('con2 = y2 - 24.0'))
+        self.add_subsystem('con_cmp1', om.ExecComp('con1 = 3.16 - y1'))
+        self.add_subsystem('con_cmp2', om.ExecComp('con2 = y2 - 24.0'))
 
         self.connect('dv.x', ['d1.x', 'obj_cmp.x'])
         self.connect('dv.y1', ['d2.y1', 'obj_cmp.y1', 'con_cmp1.y1'])
@@ -265,7 +269,7 @@ class SellarIDF(Group):
         # rather than create a cycle by connecting d1.y1 to d2.y1 and d2.y2 to d1.y2
         # we will constrain y1 and y2 to be equal for the two disciplines
 
-        equal = EQConstraintComp()
+        equal = om.EQConstraintComp()
         self.add_subsystem('equal', equal)
 
         equal.add_eq_output('y1', add_constraint=True)
