@@ -2,21 +2,20 @@ from __future__ import print_function, division, absolute_import
 
 import unittest
 
+import openmdao.api as om
 from openmdao.utils.assert_utils import assert_rel_error
 
-from openmdao.api import Problem, ScipyOptimizeDriver, ExecComp, IndepVarComp, DirectSolver
-
 
 class TestSellarOpt(unittest.TestCase):
 
     def test_sellar_opt(self):
-        from openmdao.api import Problem, ScipyOptimizeDriver, ExecComp, IndepVarComp, DirectSolver
+        import openmdao.api as om
         from openmdao.test_suite.components.sellar_feature import SellarMDA
 
-        prob = Problem()
+        prob = om.Problem()
         prob.model = SellarMDA()
 
-        prob.driver = ScipyOptimizeDriver()
+        prob.driver = om.ScipyOptimizeDriver()
         prob.driver.options['optimizer'] = 'SLSQP'
         # prob.driver.options['maxiter'] = 100
         prob.driver.options['tol'] = 1e-8
