@@ -11,7 +11,10 @@ import unittest
 from fnmatch import fnmatchcase
 from six import string_types, PY2, reraise
 from six.moves import range, cStringIO as StringIO
-from collections import Iterable
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
 import numbers
 import json
 import importlib
@@ -712,6 +715,37 @@ def json_loads_byteified(json_str):
         return json.loads(json_str)
 
 
+def remove_whitespace(s, right=False, left=False):
+    """
+    Remove white-space characters from the given string.
+
+    If neither right nor left is specified (the default),
+    then all white-space is removed.
+
+    Parameters
+    ----------
+    s : str
+        The string to be modified.
+    right : bool
+        If True, remove white-space from the end of the string.
+    left : bool
+        If True, remove white-space from the beginning of the string.
+
+    Returns
+    -------
+    str
+        The string with white-space removed.
+    """
+    if not left and not right:
+        return re.sub(r"\s+", "", s, flags=re.UNICODE)
+    elif right and left:
+        return re.sub("^\s+|\s+$", "", s, flags=re.UNICODE)
+    elif right:
+        return re.sub(r"\s+$", "", s, flags=re.UNICODE)
+    else:  # left
+        return re.sub(r"^\s+", "", s, flags=re.UNICODE)
+
+
 _badtab = r'`~@#$%^&*()[]{}-+=|\/?<>,.:;'
 if PY2:
     import string
@@ -764,3 +798,74 @@ def make_serializable(o):
         return make_serializable(o.__class__.__name__)
     else:
         return o
+
+
+def make_set(str_data, name=None):
+    """
+    Construct a set containing the specified character strings.
+
+    Parameters
+    ----------
+    str_data : None, str, or list of strs
+        Character string(s) to be included in the set.
+
+    name : str, optional
+        A name to be used in error messages.
+
+    Returns
+    -------
+    set
+        A set of character strings.
+    """
+    if not str_data:
+        return set()
+    elif isinstance(str_data, str):
+        return {str_data}
+    elif isinstance(str_data, set):
+        return str_data
+    elif isinstance(str_data, list):
+        return set(str_data)
+    elif name:
+        raise TypeError("The {} argument should be str, set, or list: {}".format(name, str_data))
+    else:
+        raise TypeError("The argument should be str, set, or list: {}".format(str_data))
+
+
+def var_name_match_includes_excludes(name, prom_name, includes, excludes):
+    """
+    Check to see if the name passes thru the includes and excludes filter.
+
+    Parameters
+    ----------
+    name : str
+        Unpromoted var name to be checked for match.
+    prom_name : str
+        Promoted var name to be checked for match.
+    includes : None or list_like
+        List of glob patterns for name to include in the filtering.
+    excludes : None or list_like
+        List of glob patterns for name to exclude in the filtering.
+
+    Returns
+    -------
+    bool
+        Return True if the name passes through the filtering of includes and excludes.
+    """
+    # Process includes
+    if includes is not None:
+        for pattern in includes:
+            if fnmatchcase(name, pattern) or fnmatchcase(prom_name, pattern):
+                break
+        else:  # didn't find any match
+            return False
+
+    # Process excludes
+    if excludes is not None:
+        match = False
+        for pattern in excludes:
+            if fnmatchcase(name, pattern) or fnmatchcase(prom_name, pattern):
+                match = True
+                break
+        return not match
+
+    return True
