@@ -4,7 +4,8 @@ import os
 import unittest
 
 # from openmdao.utils.assert_utils import assert_rel_error
-from openmdao.utils.units import NumberDict, PhysicalUnit, _find_unit, import_library, add_unit, add_offset_unit
+from openmdao.utils.units import NumberDict, PhysicalUnit, _find_unit, import_library, \
+    add_unit, add_offset_unit, get_conversion
 
 
 class TestNumberDict(unittest.TestCase):
@@ -244,6 +245,16 @@ class TestPhysicalUnit(unittest.TestCase):
         y = x2 / (x1**2)
         self.assertEqual(y.name(), 'kg/m**2')
 
+    def test_get_conversion(self):
+        self.assertEqual(get_conversion('km', 'm'), (1000., 0.))
+
+        try:
+            get_conversion('km', 1.0)
+        except RuntimeError as err:
+            self.assertEqual(str(err), "Cannot convert to new units: 1.0")
+        else:
+            self.fail("Expecting RuntimeError")
+
 
 class TestModuleFunctions(unittest.TestCase):
     def test_add_unit(self):
