@@ -155,20 +155,11 @@ class TestPETScVector2Proc(unittest.TestCase):
 
         assert_rel_error(self, norm_val, 0.22595230821097395, 1e-10)
 
-    def test_dot_petsc(self):
-        p = om.Problem()
-        comp = om.IndepVarComp()
-        comp.add_output('v1', val=1.0)
-        comp.add_output('v2', val=2.0)
-        p.model.add_subsystem('des_vars', comp, promotes=['*'])
-        p.setup()
-        p.final_setup()
-
-        new_vec = p.model._outputs._clone()
-        new_vec.set_const(3.)
-
-        self.assertEqual(new_vec.dot(p.model._outputs), 9.)
-
+        # test petsc dot while we're at it
+        vec.set_const(3.)
+        vec2 = prob.model._vectors['residual']['linear']
+        vec2.set_const(4.)
+        assert_rel_error(self, vec.dot(vec2), 12.*6, 1e-10)
 
 @unittest.skipUnless(MPI and PETScVector, "only run with MPI and PETSc.")
 class TestPETScVector3Proc(unittest.TestCase):
@@ -196,6 +187,12 @@ class TestPETScVector3Proc(unittest.TestCase):
 
         assert_rel_error(self, norm_val, 0.22595230821097395, 1e-10)
 
+        # test petsc dot while we're at it
+        vec.set_const(3.)
+        vec2 = prob.model._vectors['residual']['linear']
+        vec2.set_const(4.)
+        assert_rel_error(self, vec.dot(vec2), 12.*6, 1e-10)
+
     def test_distributed_norm_parallel_group(self):
         prob = om.Problem()
         model = prob.model
@@ -230,6 +227,12 @@ class TestPETScVector3Proc(unittest.TestCase):
         norm_val = vec.get_norm()
         assert_rel_error(self, norm_val, 8.888194417315589, 1e-10)
 
+        # test petsc dot while we're at it
+        vec.set_const(3.)
+        vec2 = prob.model._vectors['residual']['linear']
+        vec2.set_const(4.)
+        assert_rel_error(self, vec.dot(vec2), 12.*13, 1e-10)
+
 
 if __name__ == '__main__':
     unittest.main()
