@@ -60,6 +60,8 @@ class Vector(object):
         If True, then space for the complex vector is also allocated.
     _data : ndarray
         Actual allocated data.
+    _slices : dict
+        Mapping of var name to slice.
     _cplx_data : ndarray
         Actual allocated data under complex step.
     _cplx_views : dict
@@ -134,6 +136,7 @@ class Vector(object):
 
         self._root_vector = None
         self._data = None
+        self._slices = None
 
         # Support for Complex Step
         self._alloc_complex = alloc_complex
@@ -350,7 +353,8 @@ class Vector(object):
         root_vector : <Vector> or None
             the root's vector instance or None, if we are at the root.
         """
-        pass
+        raise NotImplementedError('_initialize_data not defined for vector type %s' %
+                                  type(self).__name__)
 
     def _initialize_views(self):
         """
@@ -363,7 +367,8 @@ class Vector(object):
         - _views
         - _views_flat
         """
-        pass
+        raise NotImplementedError('_initialize_views not defined for vector type %s' %
+                                  type(self).__name__)
 
     def _clone_data(self):
         """
@@ -371,7 +376,8 @@ class Vector(object):
 
         Must be implemented by the subclass.
         """
-        pass
+        raise NotImplementedError('_clone_data not defined for vector type %s' %
+                                  type(self).__name__)
 
     def __iadd__(self, vec):
         """
@@ -384,7 +390,8 @@ class Vector(object):
         vec : <Vector>
             vector to add to self.
         """
-        pass
+        raise NotImplementedError('__iadd__ not defined for vector type %s' %
+                                  type(self).__name__)
 
     def __isub__(self, vec):
         """
@@ -397,7 +404,8 @@ class Vector(object):
         vec : <Vector>
             vector to subtract from self.
         """
-        pass
+        raise NotImplementedError('__isub__ not defined for vector type %s' %
+                                  type(self).__name__)
 
     def __imul__(self, val):
         """
@@ -410,7 +418,8 @@ class Vector(object):
         val : int or float
             scalar to multiply self.
         """
-        pass
+        raise NotImplementedError('__imul__ not defined for vector type %s' %
+                                  type(self).__name__)
 
     def add_scal_vec(self, val, vec):
         """
@@ -425,7 +434,8 @@ class Vector(object):
         vec : <Vector>
             this vector times val is added to self.
         """
-        pass
+        raise NotImplementedError('add_scale_vec not defined for vector type %s' %
+                                  type(self).__name__)
 
     def scale(self, scale_to):
         """
@@ -457,7 +467,8 @@ class Vector(object):
         vec : <Vector>
             the vector whose values self is set to.
         """
-        pass
+        raise NotImplementedError('set_vec not defined for vector type %s' %
+                                  type(self).__name__)
 
     def set_const(self, val):
         """
@@ -470,7 +481,8 @@ class Vector(object):
         val : int or float
             scalar to set self to.
         """
-        pass
+        raise NotImplementedError('set_const not defined for vector type %s' %
+                                  type(self).__name__)
 
     def dot(self, vec):
         """
@@ -483,7 +495,8 @@ class Vector(object):
         vec : <Vector>
             The incoming vector being dotted with self.
         """
-        pass
+        raise NotImplementedError('dot not defined for vector type %s' %
+                                  type(self).__name__)
 
     def get_norm(self):
         """
@@ -496,7 +509,9 @@ class Vector(object):
         float
             norm of this vector.
         """
-        pass
+        raise NotImplementedError('get_norm not defined for vector type %s' %
+                                  type(self).__name__)
+        return None  # silence lint warning about missing return value.
 
     def _enforce_bounds_vector(self, du, alpha, lower_bounds, upper_bounds):
         """
@@ -515,7 +530,8 @@ class Vector(object):
         upper_bounds : <Vector>
             Upper bounds vector.
         """
-        pass
+        raise NotImplementedError('_enforce_bounds_vector not defined for vector type %s' %
+                                  type(self).__name__)
 
     def _enforce_bounds_scalar(self, du, alpha, lower_bounds, upper_bounds):
         """
@@ -534,7 +550,8 @@ class Vector(object):
         upper_bounds : <Vector>
             Upper bounds vector.
         """
-        pass
+        raise NotImplementedError('_enforce_bounds_scalar not defined for vector type %s' %
+                                  type(self).__name__)
 
     def _enforce_bounds_wall(self, du, alpha, lower_bounds, upper_bounds):
         """
@@ -553,7 +570,8 @@ class Vector(object):
         upper_bounds : <Vector>
             Upper bounds vector.
         """
-        pass
+        raise NotImplementedError('_enforce_bounds_wall not defined for vector type %s' %
+                                  type(self).__name__)
 
     def set_complex_step_mode(self, active, keep_real=False):
         """
